@@ -18,25 +18,26 @@
  ****************************************************************/
 package org.apache.cayenne.crypto;
 
-import org.apache.cayenne.ObjectContext;
-import org.apache.cayenne.crypto.db.Table1;
-import org.apache.cayenne.crypto.db.Table2;
-import org.apache.cayenne.crypto.transformer.value.IntegerConverter;
-import org.apache.cayenne.crypto.unit.CryptoUnitUtils;
-import org.apache.cayenne.exp.Property;
-import org.apache.cayenne.query.ObjectSelect;
-import org.apache.cayenne.query.SelectQuery;
-import org.junit.Before;
-import org.junit.Test;
-
 import java.sql.SQLException;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
+import org.apache.cayenne.ObjectContext;
+import org.apache.cayenne.crypto.db.Table1;
+import org.apache.cayenne.crypto.db.Table2;
+import org.apache.cayenne.crypto.db.Table7;
+import org.apache.cayenne.crypto.transformer.value.IntegerConverter;
+import org.apache.cayenne.crypto.unit.CryptoUnitUtils;
+import org.apache.cayenne.exp.property.PropertyFactory;
+import org.apache.cayenne.query.ObjectSelect;
+import org.junit.Before;
+import org.junit.Test;
+
 import static org.junit.Assert.assertArrayEquals;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNull;
+import static org.junit.Assert.assertTrue;
 
 public class Runtime_AES128_IT extends Runtime_AES128_Base {
 
@@ -128,8 +129,8 @@ public class Runtime_AES128_IT extends Runtime_AES128_Base {
 
         context.commitChanges();
 
-        SelectQuery<Table2> select = SelectQuery.query(Table2.class);
-        select.addOrdering(Table2.PLAIN_BYTES.asc());
+        ObjectSelect<Table2> select = ObjectSelect.query(Table2.class)
+                .orderBy(Table2.PLAIN_BYTES.asc());
 
         List<Table2> result = runtime.newContext().select(select);
 
@@ -151,7 +152,7 @@ public class Runtime_AES128_IT extends Runtime_AES128_Base {
 
         context.commitChanges();
 
-        List<Table1> result = SelectQuery.query(Table1.class).select(runtime.newContext());
+        List<Table1> result = ObjectSelect.query(Table1.class).select(runtime.newContext());
 
         assertEquals(1, result.size());
         assertEquals(59, result.get(0).getPlainInt());
@@ -169,7 +170,7 @@ public class Runtime_AES128_IT extends Runtime_AES128_Base {
         context.commitChanges();
 
         List<Table1> result = ObjectSelect
-                .columnQuery(Table1.class, Property.createSelf(Table1.class))
+                .columnQuery(Table1.class, PropertyFactory.createSelf(Table1.class))
                 .select(context);
 
         assertEquals(1, result.size());
@@ -189,7 +190,9 @@ public class Runtime_AES128_IT extends Runtime_AES128_Base {
         context.commitChanges();
 
         List<Object[]> result = ObjectSelect
-                .columnQuery(Table1.class, Property.createSelf(Table1.class), Table1.PLAIN_INT)
+                .columnQuery(Table1.class,
+                        PropertyFactory.createSelf(Table1.class),
+                        Table1.PLAIN_INT)
                 .select(context);
 
         assertEquals(1, result.size());
@@ -207,10 +210,13 @@ public class Runtime_AES128_IT extends Runtime_AES128_Base {
         t1.setCryptoInt(1);
         t1.setPlainInt(2);
         t1.setCryptoString("test");
+        t1.setPlainString("Test");
         context.commitChanges();
 
         List<Object[]> result = ObjectSelect
-                .columnQuery(Table1.class, Property.createSelf(Table1.class), Table1.CRYPTO_INT)
+                .columnQuery(Table1.class,
+                        PropertyFactory.createSelf(Table1.class),
+                        Table1.CRYPTO_INT)
                 .select(context);
 
         assertEquals(1, result.size());
@@ -219,6 +225,97 @@ public class Runtime_AES128_IT extends Runtime_AES128_Base {
         assertEquals(1, result.get(0)[1]);
     }
 
+    @Test
+    public void testColumnQueryWithRelationshipWithTheSameNames() {
+        ObjectContext context = runtime.newContext();
+
+        Table1 t1 = context.newObject(Table1.class);
+        t1.setCryptoInt(1);
+        t1.setPlainInt(3);
+        t1.setCryptoString("test");
+        t1.setPlainString("Test");
+
+        Table7 t7 = context.newObject(Table7.class);
+        t7.setCryptoInt(2);
+        t7.setCryptoString("string");
+
+        t1.addToTable7s(t7);
+
+        context.commitChanges();
+
+        List<Object[]> result = ObjectSelect
+                .columnQuery(Table1.class,
+                        Table1.CRYPTO_INT,
+                        PropertyFactory.createSelf(Table1.class),
+                        Table1.CRYPTO_INT,
+                        Table1.TABLE7S.dot(Table7.CRYPTO_INT),
+                        Table1.TABLE7S.dot(Table7.CRYPTO_STRING))
+                .select(context);
+        assertEquals(1, result.size());
+        assertEquals(1, result.get(0)[0]);
+        assertEquals(1, ((Table1)result.get(0)[1]).getCryptoInt());
+        assertEquals("test", ((Table1)result.get(0)[1]).getCryptoString());
+        assertEquals(1, result.get(0)[2]);
+        assertEquals(2, result.get(0)[3]);
+        assertEquals("string", result.get(0)[4]);
+    }
+
+    @Test
+    public void testSelectWith2Objects() {
+        ObjectContext context = runtime.newContext();
+
+        Table1 t1 = context.newObject(Table1.class);
+        t1.setCryptoInt(1);
+        t1.setPlainInt(3);
+        t1.setCryptoString("test");
+        t1.setPlainString("Test");
+
+        Table7 t7 = context.newObject(Table7.class);
+        t7.setCryptoInt(2);
+        t7.setCryptoString("string");
+
+        t1.addToTable7s(t7);
+
+        context.commitChanges();
+
+        List<Object[]> result = ObjectSelect
+                .columnQuery(Table1.class,
+                        PropertyFactory.createSelf(Table1.class),
+                        Table1.TABLE7S.flat())
+                .select(context);
+        assertEquals(1, result.size());
+        assertEquals("test", ((Table1)result.get(0)[0]).getCryptoString());
+        assertTrue(result.get(0)[1] instanceof Table7);
+        assertEquals(2, ((Table7)result.get(0)[1]).getCryptoInt());
+    }
+
+    @Test
+    public void testObjectSelectWithPrefetch() {
+        ObjectContext context = runtime.newContext();
+
+        Table1 t1 = context.newObject(Table1.class);
+        t1.setCryptoInt(1);
+        t1.setPlainInt(3);
+        t1.setCryptoString("test");
+        t1.setPlainString("Test");
+
+        Table7 t7 = context.newObject(Table7.class);
+        t7.setCryptoInt(2);
+        t7.setCryptoString("string");
+
+        t1.addToTable7s(t7);
+
+        context.commitChanges();
+
+        List<Table1> table1s = ObjectSelect.query(Table1.class)
+                .prefetch(Table1.TABLE7S.disjoint())
+                .select(context);
+
+        assertEquals(1, table1s.size());
+        assertEquals("test", table1s.get(0).getCryptoString());
+        assertEquals("string", table1s.get(0).getTable7s().get(0).getCryptoString());
+    }
+
     @Test
     public void test_ColumnQuerySingleScalar() {
         ObjectContext context = runtime.newContext();
