@@ -7,7 +7,7 @@
  *  "License"); you may not use this file except in compliance
  *  with the License.  You may obtain a copy of the License at
  *
- *    http://www.apache.org/licenses/LICENSE-2.0
+ *    https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing,
  *  software distributed under the License is distributed on an
@@ -158,7 +158,7 @@ class HierarchicalObjectResolver {
                     createDisjointByIdPrefetchQualifier(pathPrefix, currentQuery, joins, values);
 
                     currentQuery = new PrefetchSelectQuery<>(node.getPath(), relationship);
-                    currentQuery.setFetchingDataRows(true);
+                    currentQuery.fetchDataRows();
                     queries.add(currentQuery);
                     qualifiersCount = 0;
                     values = new HashSet<>();
@@ -203,7 +203,7 @@ class HierarchicalObjectResolver {
             return true;
         }
 
-        private void createDisjointByIdPrefetchQualifier(String pathPrefix, PrefetchSelectQuery currentQuery,
+        private void createDisjointByIdPrefetchQualifier(String pathPrefix, PrefetchSelectQuery<?> currentQuery,
                                                          List<DbJoin> joins, Set<List<Object>> values) {
             Expression allJoinsQualifier;
             if(currentQuery != null) {
@@ -223,7 +223,7 @@ class HierarchicalObjectResolver {
                     qualifiers[i++] = allJoinsQualifier;
                 }
 
-                currentQuery.orQualifier(ExpressionFactory.joinExp(Expression.OR, qualifiers));
+                currentQuery.or(ExpressionFactory.joinExp(Expression.OR, qualifiers));
             }
         }
 
@@ -285,48 +285,43 @@ class HierarchicalObjectResolver {
 
             // delegate processing of the top level joint prefetch to a joint processor,
             // skip non-top joint nodes
-
-            if (node.getParent() != null && !node.getParent().isJointPrefetch()) {
-
-                PrefetchProcessorJointNode processorNode = (PrefetchProcessorJointNode) node;
-
-                JointProcessor subprocessor = new JointProcessor(processorNode);
-
-                PrefetchProcessorNode parent = (PrefetchProcessorNode) processorNode
-                        .getParent();
-
-                while (parent != null && parent.isPhantom()) {
-                    parent = (PrefetchProcessorNode) parent.getParent();
-                }
-
-                if (parent == null) {
-                    return false;
-                }
-
-                List parentRows = parent.getDataRows();
-
-                // phantom node?
-                if (parentRows == null || parentRows.size() == 0) {
-                    return false;
-                }
-
-                List<Persistent> parentObjects = parent.getObjects();
-                int size = parentRows.size();
-
-                for (int i = 0; i < size; i++) {
-                    subprocessor.setCurrentFlatRow((DataRow) parentRows.get(i));
-                    parent.setLastResolved(parentObjects.get(i));
-                    processorNode.traverse(subprocessor);
-                }
-
-                List<Persistent> objects = processorNode.getObjects();
-
-                cache.snapshotsUpdatedForObjects(
-                        objects,
-                        processorNode.getResolvedRows(),
-                        queryMetadata.isRefreshingObjects());
-
+            if (node.getParent() == null || node.getParent().isJointPrefetch()) {
+                return true;
             }
+
+            PrefetchProcessorJointNode processorNode = (PrefetchProcessorJointNode) node;
+            JointProcessor subprocessor = new JointProcessor(processorNode);
+
+            PrefetchProcessorNode parent = (PrefetchProcessorNode) processorNode.getParent();
+            while (parent != null && parent.isPhantom()) {
+                parent = (PrefetchProcessorNode) parent.getParent();
+            }
+
+            if (parent == null) {
+                return false;
+            }
+
+            List<DataRow> parentRows = parent.getDataRows();
+            // phantom node?
+            if (parentRows == null || parentRows.size() == 0) {
+                return false;
+            }
+
+            List<Persistent> parentObjects = parent.getObjects();
+            int size = parentRows.size();
+
+            for (int i = 0; i < size; i++) {
+                subprocessor.setCurrentFlatRow(parentRows.get(i));
+                parent.setLastResolved(parentObjects.get(i));
+                processorNode.traverse(subprocessor);
+            }
+
+            List<Persistent> objects = processorNode.getObjects();
+            cache.snapshotsUpdatedForObjects(
+                    objects,
+                    processorNode.getResolvedRows(),
+                    queryMetadata.isRefreshingObjects());
+
             return true;
         }
 
@@ -346,14 +341,11 @@ class HierarchicalObjectResolver {
 
             // TODO: see TODO in ObjectResolver.relatedObjectsFromDataRows
 
-            if ((node.isDisjointPrefetch() || node.isDisjointByIdPrefetch())
-                    && !needToSaveDuplicates) {
+            if ((node.isDisjointPrefetch() || node.isDisjointByIdPrefetch()) && !needToSaveDuplicates) {
                 PrefetchProcessorNode processorNode = (PrefetchProcessorNode) node;
                 if (processorNode.isJointChildren()) {
                     List<Persistent> objects = processorNode.getObjects();
-
                     if (objects != null && objects.size() > 1) {
-
                         Set<Persistent> seen = new HashSet<>(objects.size());
                         objects.removeIf(persistent -> !seen.add(persistent));
                     }
@@ -394,7 +386,7 @@ class HierarchicalObjectResolver {
             PrefetchProcessorJointNode processorNode = (PrefetchProcessorJointNode) node;
 
             // find existing object, if found skip further processing
-            Map id = processorNode.idFromFlatRow(currentFlatRow);
+            Map<String, Object> id = processorNode.idFromFlatRow(currentFlatRow);
             Persistent object = processorNode.getResolved(id);
             DataRow row = null;
             if (object == null) {
