@@ -25,9 +25,9 @@ import org.apache.cayenne.ResultIterator;
 import org.apache.cayenne.exp.Expression;
 import org.apache.cayenne.exp.ExpressionFactory;
 import org.apache.cayenne.map.ObjEntity;
+import org.apache.cayenne.query.ObjectSelect;
 import org.apache.cayenne.query.Query;
 import org.apache.cayenne.query.QueryMetadata;
-import org.apache.cayenne.query.SelectQuery;
 import org.apache.cayenne.util.Util;
 
 import java.io.Serializable;
@@ -62,7 +62,6 @@ public class IncrementalFaultList<E> implements List<E>, Serializable {
 	protected final List elements;
 	protected DataContext dataContext;
 	protected ObjEntity rootEntity;
-	protected SelectQuery<?> internalQuery;
 	protected int unfetchedObjects;
 
 	/**
@@ -72,6 +71,7 @@ public class IncrementalFaultList<E> implements List<E>, Serializable {
 	protected int idWidth;
 
 	IncrementalListHelper helper;
+	protected QueryMetadata metadata;
 
 	/**
 	 * Defines the upper limit on the size of fetches. This is needed to avoid
@@ -102,7 +102,7 @@ public class IncrementalFaultList<E> implements List<E>, Serializable {
 	 *            maximum number of fetches in one query
 	 */
 	public IncrementalFaultList(DataContext dataContext, Query query, int maxFetchSize) {
-		QueryMetadata metadata = query.getMetaData(dataContext.getEntityResolver());
+		this.metadata = query.getMetaData(dataContext.getEntityResolver());
 		if (metadata.getPageSize() <= 0) {
 			throw new CayenneRuntimeException("Not a paginated query; page size: " + metadata.getPageSize());
 		}
@@ -115,14 +115,6 @@ public class IncrementalFaultList<E> implements List<E>, Serializable {
 			throw new CayenneRuntimeException("Pagination is not supported for queries not rooted in an ObjEntity");
 		}
 
-		// create an internal query, it is a partial replica of
-		// the original query and will serve as a value holder for
-		// various parameters
-		this.internalQuery = new SelectQuery<>(rootEntity);
-		this.internalQuery.setFetchingDataRows(metadata.isFetchingDataRows());
-		this.internalQuery.setPrefetchTree(metadata.getPrefetchTree());
-
-		this.helper = createHelper(metadata);
 		this.idWidth = metadata.getDbEntity().getPrimaryKeys().size();
 
 		List<Object> elementsUnsynced = new ArrayList<>();
@@ -143,11 +135,11 @@ public class IncrementalFaultList<E> implements List<E>, Serializable {
 		}
 	}
 
-	/**
-	 * @since 1.2
-	 */
-	SelectQuery getInternalQuery() {
-		return internalQuery;
+	IncrementalListHelper getHelper() {
+		if(helper == null) {
+			helper = createHelper(metadata);
+		}
+		return helper;
 	}
 
 	/**
@@ -160,7 +152,7 @@ public class IncrementalFaultList<E> implements List<E>, Serializable {
 
 		elementsList.clear();
 
-		try (ResultIterator it = dataContext.performIteratedQuery(query)) {
+		try (ResultIterator<?> it = dataContext.performIteratedQuery(query)) {
 			while (it.hasNextRow()) {
 				elementsList.add(it.nextRow());
 			}
@@ -184,7 +176,7 @@ public class IncrementalFaultList<E> implements List<E>, Serializable {
 
 		// I am not sure if such a check makes sense???
 
-		if (internalQuery.isFetchingDataRows()) {
+		if (metadata.isFetchingDataRows()) {
 			if (!(object instanceof Map)) {
 				throw new IllegalArgumentException("Only Map objects can be stored in this list.");
 			}
@@ -223,7 +215,7 @@ public class IncrementalFaultList<E> implements List<E>, Serializable {
 			List<Object> ids = new ArrayList<>(pageSize);
 			for (int i = fromIndex; i < toIndex; i++) {
 				Object object = elements.get(i);
-				if (helper.unresolvedSuspect(object)) {
+				if (getHelper().unresolvedSuspect(object)) {
 					quals.add(buildIdQualifier(object));
 					ids.add(object);
 				}
@@ -242,7 +234,7 @@ public class IncrementalFaultList<E> implements List<E>, Serializable {
 			int fetchEnd = Math.min(qualsSize, fetchSize);
 			int fetchBegin = 0;
 			while (fetchBegin < qualsSize) {
-				SelectQuery<Object> query = createSelectQuery(quals.subList(fetchBegin, fetchEnd));
+				ObjectSelect<Persistent> query = createSelectQuery(quals.subList(fetchBegin, fetchEnd));
 				objects.addAll(dataContext.performQuery(query));
 				fetchBegin = fetchEnd;
 				fetchEnd += Math.min(fetchSize, qualsSize - fetchEnd);
@@ -258,18 +250,21 @@ public class IncrementalFaultList<E> implements List<E>, Serializable {
 
 	void updatePageWithResults(List<Object> objects, int fromIndex, int toIndex) {
 		for (Object object : objects) {
-			helper.updateWithResolvedObjectInRange(object, fromIndex, toIndex);
+			getHelper().updateWithResolvedObjectInRange(object, fromIndex, toIndex);
 		}
 
 		unfetchedObjects -= objects.size();
 	}
 
-	SelectQuery<Object> createSelectQuery(List<Expression> expressions) {
-		SelectQuery<Object> query = new SelectQuery<>(rootEntity, ExpressionFactory.joinExp(Expression.OR, expressions));
+	ObjectSelect<Persistent> createSelectQuery(List<Expression> expressions) {
+		ObjectSelect<Persistent> query = ObjectSelect.query(Persistent.class)
+				.entityName(rootEntity.getName())
+				.where(ExpressionFactory.joinExp(Expression.OR, expressions));
 
-		query.setFetchingDataRows(internalQuery.isFetchingDataRows());
-		if (!query.isFetchingDataRows()) {
-			query.setPrefetchTree(internalQuery.getPrefetchTree());
+		if(metadata.isFetchingDataRows()) {
+			query.fetchDataRows();
+		} else if (metadata.getPrefetchTree() != null) {
+			query.prefetch(metadata.getPrefetchTree());
 		}
 
 		return query;
@@ -294,39 +289,42 @@ public class IncrementalFaultList<E> implements List<E>, Serializable {
 	 * @since 3.0
 	 */
 	void checkPageResultConsistency(List<?> objects, List<?> ids) {
+		if (objects.size() == ids.size()) {
+			return;
+		} else if (objects.size() > ids.size()) {
+			throw new CayenneRuntimeException("Expected %d objects, retrieved %d", ids.size(), objects.size());
+		}
 
-		if (objects.size() < ids.size()) {
-			// find missing ids
-			StringBuilder buffer = new StringBuilder();
-			buffer.append("Some ObjectIds are missing from the database. ");
-			buffer.append("Expected ").append(ids.size()).append(", fetched ").append(objects.size());
-
-			boolean first = true;
-			for (Object id : ids) {
-				boolean found = false;
-
-				for (Object object : objects) {
-
-					if (helper.replacesObject(object, id)) {
-						found = true;
-						break;
-					}
-				}
-
-				if (!found) {
-					if (first) {
-						first = false;
-					} else {
-						buffer.append(", ");
-					}
-
-					buffer.append(id.toString());
+		// We have less objects then ids
+		// check that we are really missing some ids and throw an exception in that case
+		StringBuilder buffer = null;
+		boolean first = true;
+		for (Object id : ids) {
+			boolean found = false;
+			for (Object object : objects) {
+				if (getHelper().replacesObject(object, id)) {
+					found = true;
+					break;
 				}
 			}
 
+			if (!found) {
+				if(buffer == null) {
+					buffer = new StringBuilder();
+				}
+				if (first) {
+					first = false;
+				} else {
+					buffer.append(", ");
+				}
+				buffer.append(id.toString());
+			}
+		}
+
+		// we have some objects missing, throw
+		if(buffer != null) {
+			buffer.insert(0, "Some ObjectIds are missing from the database. Expected " + ids.size() + ", fetched " + objects.size());
 			throw new CayenneRuntimeException(buffer.toString());
-		} else if (objects.size() > ids.size()) {
-			throw new CayenneRuntimeException("Expected %d objects, retrieved %d", ids.size(), objects.size());
 		}
 	}
 
@@ -506,7 +504,7 @@ public class IncrementalFaultList<E> implements List<E>, Serializable {
 		synchronized (elements) {
 			Object o = elements.get(index);
 
-			if (helper.unresolvedSuspect(o)) {
+			if (getHelper().unresolvedSuspect(o)) {
 				// read this page
 				int pageStart = pageIndex(index) * pageSize;
 				resolveInterval(pageStart, pageStart + pageSize);
@@ -522,7 +520,7 @@ public class IncrementalFaultList<E> implements List<E>, Serializable {
 	 * @see java.util.List#indexOf(Object)
 	 */
 	public int indexOf(Object o) {
-		return helper.indexOfObject(o);
+		return getHelper().indexOfObject(o);
 	}
 
 	/**
@@ -535,7 +533,7 @@ public class IncrementalFaultList<E> implements List<E>, Serializable {
 	}
 
 	public int lastIndexOf(Object o) {
-		return helper.lastIndexOfObject(o);
+		return getHelper().lastIndexOfObject(o);
 	}
 
 	public E remove(int index) {
