@@ -19,6 +19,7 @@
 
 package org.apache.cayenne.access.jdbc;
 
+import org.apache.cayenne.ObjectId;
 import org.apache.cayenne.ResultIterator;
 import org.apache.cayenne.access.DataNode;
 import org.apache.cayenne.access.OperationObserver;
@@ -30,7 +31,6 @@ import org.apache.cayenne.dba.DbAdapter;
 import org.apache.cayenne.dba.TypesMapping;
 import org.apache.cayenne.log.JdbcEventLogger;
 import org.apache.cayenne.map.DbAttribute;
-import org.apache.cayenne.map.ObjAttribute;
 import org.apache.cayenne.query.BatchQuery;
 import org.apache.cayenne.query.BatchQueryRow;
 import org.apache.cayenne.query.InsertBatchQuery;
@@ -40,8 +40,10 @@ import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
+import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.List;
 
 /**
  * @since 1.2
@@ -53,7 +55,7 @@ public class BatchAction extends BaseSQLAction {
 	protected RowDescriptor keyRowDescriptor;
 
 	private static void bind(DbAdapter adapter, PreparedStatement statement, DbAttributeBinding[] bindings)
-			throws SQLException, Exception {
+			throws Exception {
 
 		for (DbAttributeBinding b : bindings) {
 			if (!b.isExcluded()) {
@@ -79,24 +81,39 @@ public class BatchAction extends BaseSQLAction {
 	}
 
 	@Override
-	public void performAction(Connection connection, OperationObserver observer) throws SQLException, Exception {
-
+	public void performAction(Connection connection, OperationObserver observer) throws Exception {
 		BatchTranslator translator = createTranslator();
-		boolean generatesKeys = hasGeneratedKeys();
 
-		if (runningAsBatch && !generatesKeys) {
-			runAsBatch(connection, translator, observer);
+		boolean isBatch = canRunAsBatch();
+		boolean generatesKeys = hasGeneratedKeys() && supportsGeneratedKeys(isBatch);
+
+		if (isBatch) {
+			runAsBatch(connection, translator, observer, generatesKeys);
 		} else {
 			runAsIndividualQueries(connection, translator, observer, generatesKeys);
 		}
 	}
 
+	protected boolean canRunAsBatch() {
+		if(!runningAsBatch || query.getRows().size() <= 1) {
+			return false;
+		}
+
+		if (hasGeneratedKeys()) {
+			// turn off batch mode if we generate keys but can't do so in a batch
+			return supportsGeneratedKeys(true) &&
+					!dataNode.getEntityResolver().getEntitySorter().isReflexive(query.getDbEntity());
+		}
+
+		return true;
+	}
+
 	protected BatchTranslator createTranslator() {
 		return dataNode.batchTranslator(query, null);
 	}
 
-	protected void runAsBatch(Connection con, BatchTranslator translator, OperationObserver delegate)
-			throws SQLException, Exception {
+	protected void runAsBatch(Connection con, BatchTranslator translator, OperationObserver delegate, boolean generatesKeys)
+			throws Exception {
 
 		String sql = translator.getSql();
 		JdbcEventLogger logger = dataNode.getJdbcEventLogger();
@@ -109,7 +126,7 @@ public class BatchAction extends BaseSQLAction {
 
 		DbAdapter adapter = dataNode.getAdapter();
 
-		try (PreparedStatement statement = con.prepareStatement(sql)) {
+		try (PreparedStatement statement = prepareStatement(con, sql, adapter, generatesKeys)) {
 			for (BatchQueryRow row : query.getRows()) {
 
 				DbAttributeBinding[] bindings = translator.updateBindings(row);
@@ -123,6 +140,10 @@ public class BatchAction extends BaseSQLAction {
 			int[] results = statement.executeBatch();
 			delegate.nextBatchCount(query, results);
 
+			if (generatesKeys) {
+				processGeneratedKeys(statement, delegate, query.getRows());
+			}
+			
 			if (isLoggable) {
 				int totalUpdateCount = 0;
 				for (int result : results) {
@@ -148,6 +169,10 @@ public class BatchAction extends BaseSQLAction {
 	protected void runAsIndividualQueries(Connection connection, BatchTranslator translator,
 			OperationObserver delegate, boolean generatesKeys) throws SQLException, Exception {
 
+		if(query.getRows().isEmpty()) {
+			return;
+		}
+
 		JdbcEventLogger logger = dataNode.getJdbcEventLogger();
 		boolean useOptimisticLock = query.isUsingOptimisticLocking();
 
@@ -192,15 +217,17 @@ public class BatchAction extends BaseSQLAction {
 				: connection.prepareStatement(queryStr);
 	}
 
+	protected boolean supportsGeneratedKeys(boolean isBatch) {
+		// see if we are configured to support generated keys
+		return isBatch
+				? dataNode.getAdapter().supportsGeneratedKeysForBatchInserts()
+				: dataNode.getAdapter().supportsGeneratedKeys();
+	}
+				
 	/**
 	 * Returns whether BatchQuery generates any keys.
 	 */
 	protected boolean hasGeneratedKeys() {
-		// see if we are configured to support generated keys
-		if (!dataNode.getAdapter().supportsGeneratedKeys()) {
-			return false;
-		}
-
 		// see if the query needs them
 		if (query instanceof InsertBatchQuery) {
 
@@ -220,9 +247,17 @@ public class BatchAction extends BaseSQLAction {
 	 * 
 	 * @since 4.0
 	 */
-	@SuppressWarnings({ "rawtypes", "unchecked" })
 	protected void processGeneratedKeys(Statement statement, OperationObserver observer, BatchQueryRow row)
 			throws SQLException {
+		processGeneratedKeys(statement, observer, Collections.singletonList(row));
+	}
+
+	/**
+	 * @since 4.2
+	 */
+	@SuppressWarnings({ "rawtypes", "unchecked" })
+	protected void processGeneratedKeys(Statement statement, OperationObserver observer, List<BatchQueryRow> rows)
+			throws SQLException {
 
 		ResultSet keysRS = statement.getGeneratedKeys();
 
@@ -240,17 +275,15 @@ public class BatchAction extends BaseSQLAction {
 		if (this.keyRowDescriptor == null) {
 			// attempt to figure out the right descriptor from the mapping...
 			Collection<DbAttribute> generated = query.getDbEntity().getGeneratedAttributes();
-			if (generated.size() == 1) {
+			if (generated.size() == 1 && keysRS.getMetaData().getColumnCount() == 1) {
 				DbAttribute key = generated.iterator().next();
 
 				ColumnDescriptor[] columns = new ColumnDescriptor[1];
 
-				// use column name from result set, but type and Java class from
-				// DB
-				// attribute
+				// use column name from result set, but type and Java class from DB attribute
 				columns[0] = new ColumnDescriptor(keysRS.getMetaData(), 1);
 				columns[0].setJdbcType(key.getType());
-				columns[0].setJavaClass(TypesMapping.getJavaBySqlType(key.getType()));
+				columns[0].setJavaClass(TypesMapping.getJavaBySqlType(key));
 				builder.setColumns(columns);
 			} else {
 				builder.setResultSet(keysRS);
@@ -260,9 +293,13 @@ public class BatchAction extends BaseSQLAction {
 		}
 
 		RowReader<?> rowReader = dataNode.rowReader(keyRowDescriptor, query.getMetaData(dataNode.getEntityResolver()),
-				Collections.<ObjAttribute, ColumnDescriptor> emptyMap());
+				Collections.emptyMap());
 		ResultIterator iterator = new JDBCResultIterator(null, keysRS, rowReader);
 
-		observer.nextGeneratedRows(query, iterator, row.getObjectId());
+		List<ObjectId> objectIds = new ArrayList<>(rows.size());
+		for(BatchQueryRow row : rows) {
+			objectIds.add(row.getObjectId());
+		}
+		observer.nextGeneratedRows(query, iterator, objectIds);
 	}
 }
