@@ -19,6 +19,11 @@
 
 package org.apache.cayenne.access.jdbc;
 
+import java.sql.Connection;
+import java.sql.PreparedStatement;
+import java.sql.ResultSet;
+import java.util.List;
+
 import org.apache.cayenne.ResultIterator;
 import org.apache.cayenne.access.DataNode;
 import org.apache.cayenne.access.OperationObserver;
@@ -30,12 +35,7 @@ import org.apache.cayenne.log.JdbcEventLogger;
 import org.apache.cayenne.query.PrefetchProcessor;
 import org.apache.cayenne.query.PrefetchTreeNode;
 import org.apache.cayenne.query.QueryMetadata;
-import org.apache.cayenne.query.SelectQuery;
-
-import java.sql.Connection;
-import java.sql.PreparedStatement;
-import java.sql.ResultSet;
-import java.util.List;
+import org.apache.cayenne.query.Select;
 
 /**
  * A SQLAction that handles SelectQuery execution.
@@ -65,13 +65,13 @@ public class SelectAction extends BaseSQLAction {
 
 	}
 
-	protected SelectQuery<?> query;
+	protected Select<?> query;
 	protected QueryMetadata queryMetadata;
 
 	/**
 	 * @since 4.0
 	 */
-	public SelectAction(SelectQuery<?> query, DataNode dataNode) {
+	public SelectAction(Select<?> query, DataNode dataNode) {
 		super(dataNode);
 		this.query = query;
 		this.queryMetadata = query.getMetaData(dataNode.getEntityResolver());
@@ -97,6 +97,11 @@ public class SelectAction extends BaseSQLAction {
 			statement.setFetchSize(fetchSize);
 		}
 
+		int queryTimeout = queryMetadata.getQueryTimeout();
+		if(queryTimeout != QueryMetadata.QUERY_TIMEOUT_DEFAULT) {
+			statement.setQueryTimeout(queryTimeout);
+		}
+
 		ResultSet rs;
 
 		// need to run in try-catch block to close statement properly if
@@ -163,10 +168,10 @@ public class SelectAction extends BaseSQLAction {
 		// needed, as the SQL result count does not directly correspond to the
 		// number of objects returned from Cayenne.
 
-		int fetchLimit = query.getFetchLimit();
+		int fetchLimit = queryMetadata.getFetchLimit();
 		int offset = translator.isSuppressingDistinct()
-				? query.getFetchOffset()
-				: getInMemoryOffset(query.getFetchOffset());
+				? queryMetadata.getFetchOffset()
+				: getInMemoryOffset(queryMetadata.getFetchOffset());
 
 		if (fetchLimit > 0 || offset > 0) {
 			return new LimitResultIterator<>(iterator, offset, fetchLimit);
