@@ -1,20 +1,20 @@
 /*****************************************************************
- *   Licensed to the Apache Software Foundation (ASF) under one
- *  or more contributor license agreements.  See the NOTICE file
- *  distributed with this work for additional information
- *  regarding copyright ownership.  The ASF licenses this file
- *  to you under the Apache License, Version 2.0 (the
- *  "License"); you may not use this file except in compliance
- *  with the License.  You may obtain a copy of the License at
- *
- *    http://www.apache.org/licenses/LICENSE-2.0
- *
- *  Unless required by applicable law or agreed to in writing,
- *  software distributed under the License is distributed on an
- *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
- *  KIND, either express or implied.  See the License for the
- *  specific language governing permissions and limitations
- *  under the License.
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ * <p/>
+ * https://www.apache.org/licenses/LICENSE-2.0
+ * <p/>
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  ****************************************************************/
 package org.apache.cayenne.dba;
 
@@ -24,16 +24,16 @@ import java.util.Collection;
 import java.util.List;
 
 import org.apache.cayenne.access.DataNode;
+import org.apache.cayenne.access.sqlbuilder.sqltree.SQLTreeProcessor;
 import org.apache.cayenne.access.translator.ParameterBinding;
 import org.apache.cayenne.access.translator.ejbql.EJBQLTranslatorFactory;
-import org.apache.cayenne.access.translator.select.QualifierTranslator;
-import org.apache.cayenne.access.translator.select.QueryAssembler;
 import org.apache.cayenne.access.translator.select.SelectTranslator;
 import org.apache.cayenne.access.types.ExtendedTypeMap;
 import org.apache.cayenne.map.DbAttribute;
 import org.apache.cayenne.map.DbEntity;
 import org.apache.cayenne.map.DbRelationship;
 import org.apache.cayenne.map.EntityResolver;
+import org.apache.cayenne.query.FluentSelect;
 import org.apache.cayenne.query.Query;
 import org.apache.cayenne.query.SQLAction;
 import org.apache.cayenne.query.SelectQuery;
@@ -54,14 +54,26 @@ public interface DbAdapter {
 	 */
 	String getBatchTerminator();
 
-	/**
-	 * Returns a SelectTranslator that works with the adapter target database.
-	 *
-	 * @since 4.0
-	 */
-	SelectTranslator getSelectTranslator(SelectQuery<?> query, EntityResolver entityResolver);
+    /**
+     * Returns a SelectTranslator that works with the adapter target database.
+     *
+     * @since 4.0
+     * @deprecated since 4.2 as {@link SelectQuery} is deprecated.
+     * {@link #getSelectTranslator(FluentSelect, EntityResolver)} replaces this method.
+     */
+    @Deprecated
+    SelectTranslator getSelectTranslator(SelectQuery<?> query, EntityResolver entityResolver);
 
-	QualifierTranslator getQualifierTranslator(QueryAssembler queryAssembler);
+	/**
+	 * @since 4.2
+	 */
+	SelectTranslator getSelectTranslator(FluentSelect<?> query, EntityResolver entityResolver);
+
+	/**
+	 * @since 4.2
+	 * @return {@link SQLTreeProcessor} that can adjust SQL tree to specific database flavour
+	 */
+	SQLTreeProcessor getSqlTreeProcessor();
 
 	/**
 	 * Returns an instance of SQLAction that should handle the query.
@@ -93,6 +105,15 @@ public interface DbAdapter {
 	 */
 	boolean supportsGeneratedKeys();
 
+    /**
+	 * Returns true if a target database supports key autogeneration in a batch insert.
+	 * @see #supportsGeneratedKeys()
+     * @since 4.2
+     */
+    default boolean supportsGeneratedKeysForBatchInserts() {
+    	return supportsGeneratedKeys();
+    }
+
 	/**
 	 * Returns <code>true</code> if the target database supports batch updates.
 	 */
@@ -234,4 +255,5 @@ public interface DbAdapter {
 	 * @return list of system schemas
 	 */
 	List<String> getSystemSchemas();
+
 }
