@@ -7,7 +7,7 @@
  *  "License"); you may not use this file except in compliance
  *  with the License.  You may obtain a copy of the License at
  *
- *    http://www.apache.org/licenses/LICENSE-2.0
+ *    https://www.apache.org/licenses/LICENSE-2.0
  *
  *  Unless required by applicable law or agreed to in writing,
  *  software distributed under the License is distributed on an
@@ -20,8 +20,10 @@ package org.apache.cayenne.dba.db2;
 
 import org.apache.cayenne.access.DataNode;
 import org.apache.cayenne.dba.JdbcActionBuilder;
+import org.apache.cayenne.query.FluentSelect;
 import org.apache.cayenne.query.ProcedureQuery;
 import org.apache.cayenne.query.SQLAction;
+import org.apache.cayenne.query.SelectQuery;
 
 /**
  * @since 3.1
@@ -36,4 +38,17 @@ public class DB2ActionBuilder extends JdbcActionBuilder {
     public SQLAction procedureAction(ProcedureQuery query) {
         return new DB2ProcedureAction(query, dataNode);
     }
+
+    @Override
+    public <T> SQLAction objectSelectAction(SelectQuery<T> query) {
+        return new DB2SelectAction(query, dataNode);
+    }
+
+    /**
+     * @since 4.2
+     */
+    @Override
+    public <T> SQLAction objectSelectAction(FluentSelect<T> query) {
+        return new DB2SelectAction(query, dataNode);
+    }
 }
