@@ -45,14 +45,14 @@ public class ASTLikeIgnoreCase extends IgnoreCaseNode {
 	public ASTLikeIgnoreCase(SimpleNode path, Object pattern) {
 		super(ExpressionParserTreeConstants.JJTLIKEIGNORECASE, true);
 		jjtAddChild(path, 0);
-		jjtAddChild(new ASTScalar(pattern), 1);
+		jjtAddChild(wrap(pattern), 1);
 		connectChildren();
 	}
 
 	public ASTLikeIgnoreCase(SimpleNode path, Object pattern, char escapeChar) {
 		super(ExpressionParserTreeConstants.JJTLIKEIGNORECASE, true, escapeChar);
 		jjtAddChild(path, 0);
-		jjtAddChild(new ASTScalar(pattern), 1);
+		jjtAddChild(wrap(pattern), 1);
 		connectChildren();
 	}
 
