@@ -18,6 +18,14 @@
  ****************************************************************/
 package org.apache.cayenne.query;
 
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
+import java.util.function.Function;
+
 import org.apache.cayenne.CayenneRuntimeException;
 import org.apache.cayenne.DataRow;
 import org.apache.cayenne.ObjectContext;
@@ -27,13 +35,6 @@ import org.apache.cayenne.ResultIteratorCallback;
 import org.apache.cayenne.map.DataMap;
 import org.apache.cayenne.map.EntityResolver;
 
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Collections;
-import java.util.HashMap;
-import java.util.List;
-import java.util.Map;
-
 /**
  * A selecting query based on raw SQL and featuring fluent API.
  * 
@@ -46,6 +47,7 @@ public class SQLSelect<T> extends IndirectQuery implements Select<T> {
 	private List<Class<?>> resultColumnsTypes;
 	private boolean useScalar;
 	private boolean isFetchingDataRows;
+	private Function<T, ?> resultMapper;
 
 	/**
 	 * Creates a query that selects DataRows and uses default routing.
@@ -93,7 +95,9 @@ public class SQLSelect<T> extends IndirectQuery implements Select<T> {
 
 	/**
 	 * Creates a query that selects scalar values and uses default routing.
+	 * @deprecated since 4.2. Use {@link #scalarQuery(String, Class)} method.
 	 */
+	@Deprecated
 	public static <T> SQLSelect<T> scalarQuery(Class<T> type, String sql) {
 		return scalarQuery(sql, type);
 	}
@@ -101,7 +105,9 @@ public class SQLSelect<T> extends IndirectQuery implements Select<T> {
 	/**
 	 * Creates a query that selects scalar values and uses routing based on the
 	 * provided DataMap name.
+	 * @deprecated since 4.2. Use {@link #scalarQuery(String, String, Class)} method.
 	 */
+	@Deprecated
 	public static <T> SQLSelect<T> scalarQuery(Class<T> type, String dataMapName, String sql) {
 		SQLSelect<T> query = new SQLSelect<>(sql);
 		query.dataMapName = dataMapName;
@@ -133,31 +139,69 @@ public class SQLSelect<T> extends IndirectQuery implements Select<T> {
 	 * Creates query that selects scalar value and uses default routing
 	 *
 	 * @since 4.1
+	 * @deprecated since 4.2. Use {@link #columnQuery(String)}
 	 */
+	@Deprecated
 	public static SQLSelect<Object[]> scalarQuery(String sql) {
 		SQLSelect<Object[]> query = new SQLSelect<>(sql);
 		return query.useScalar();
 	}
 
+	/**
+	 *  Creates query that selects scalar values (as Object[]) and uses default routing
+	 *
+	 *  @since 4.2
+	 */
+	public static SQLSelect<Object[]> columnQuery(String sql) {
+		SQLSelect<Object[]> query = new SQLSelect<>(sql);
+		return query.useScalar();
+	}
+
 	/**
 	 * Creates query that selects scalar values (as Object[]) and uses routing based on the
 	 * provided DataMap name.
 	 * @since 4.1
+	 * @deprecated since 4.2. Use {@link #columnQuery(String, String)}
 	 */
+	@Deprecated
 	public static SQLSelect<Object[]> scalarQuery(String sql, String dataMapName) {
 		SQLSelect<Object[]> query = new SQLSelect<>(sql);
 		query.dataMapName = dataMapName;
 		return query.useScalar();
 	}
 
+	/**
+	 * Creates query that selects scalar values (as Object[]) and uses routing based on the
+	 * provided DataMap name.
+	 *
+	 * @since 4.2
+	 */
+	public static SQLSelect<Object[]> columnQuery(String sql, String dataMapName) {
+		SQLSelect<Object[]> query = new SQLSelect<>(sql);
+		query.dataMapName = dataMapName;
+		return query.useScalar();
+	}
+
 	/**
 	 * Creates query that selects scalar values (as Object[]) and uses default routing
 	 *
 	 * @since 4.1
+	 * @deprecated since 4.2. Use {@link #columnQuery(String, Class...)}
 	 */
-	public static SQLSelect<Object[]> scalarQuery(String sql, Class<?> firstType, Class<?>... types) {
+	@Deprecated
+	public static SQLSelect<Object[]> scalarQuery(String sql, Class<?>... types) {
 		SQLSelect<Object[]> query = new SQLSelect<>(sql);
-		return query.resultColumnsTypes(firstType).resultColumnsTypes(types).useScalar();
+		return query.resultColumnsTypes(types).useScalar();
+	}
+
+	/**
+	 * Creates query that selects scalar values (as Object[]) and uses default routing
+	 *
+	 * @since 4.2
+	 */
+	public static SQLSelect<Object[]> columnQuery(String sql, Class<?>... types) {
+		SQLSelect<Object[]> query = new SQLSelect<>(sql);
+		return query.resultColumnsTypes(types).useScalar();
 	}
 
 	/**
@@ -165,15 +209,36 @@ public class SQLSelect<T> extends IndirectQuery implements Select<T> {
 	 * provided DataMap name.
 	 *
 	 * @since 4.1
+	 * @deprecated since 4.2. Use {@link #columnQuery(String, String, Class...)}
 	 */
-	public static SQLSelect<Object[]> scalarQuery(String sql, String dataMapName, Class<?> firstType, Class<?>... types) {
+	@Deprecated
+	public static SQLSelect<Object[]> scalarQuery(String sql,
+												  String dataMapName,
+												  Class<?>... types) {
 		SQLSelect<Object[]> query = new SQLSelect<>(sql);
 		query.dataMapName = dataMapName;
-		return query.resultColumnsTypes(firstType).resultColumnsTypes(types).useScalar();
+		return query.resultColumnsTypes(types).useScalar();
+	}
+
+	/**
+	 * Creates query that selects scalar values (as Object[]) and uses routing based on the
+	 * provided DataMap name.
+	 *
+	 * @since 4.2
+	 */
+	public static SQLSelect<Object[]> columnQuery(String sql,
+												  String dataMapName,
+												  Class<?>... types) {
+		SQLSelect<Object[]> query = new SQLSelect<>(sql);
+		query.dataMapName = dataMapName;
+		return query.resultColumnsTypes(types).useScalar();
 	}
 
 	@SuppressWarnings("unchecked")
 	private <E> SQLSelect<E> resultColumnsTypes(Class<?>... types) {
+		if(types.length == 0) {
+			throw new IllegalArgumentException("Empty types");
+		}
 		if(resultColumnsTypes == null) {
 			resultColumnsTypes = new ArrayList<>(types.length);
 		}
@@ -193,6 +258,7 @@ public class SQLSelect<T> extends IndirectQuery implements Select<T> {
 	protected int offset;
 	protected int pageSize;
 	protected int statementFetchSize;
+	protected int queryTimeout;
 	protected PrefetchTreeNode prefetches;
 
 	public SQLSelect(String sql) {
@@ -255,11 +321,24 @@ public class SQLSelect<T> extends IndirectQuery implements Select<T> {
 		return this;
 	}
 
+	/**
+	 * @deprecated since 4.2
+	 * Use {@link #param(String, Object)}
+	 */
+	@Deprecated
 	public SQLSelect<T> params(String name, Object value) {
 		params(Collections.singletonMap(name, value));
 		return this;
 	}
 
+	/**
+	 * @since 4.2
+	 */
+	public SQLSelect<T> param(String name, Object value) {
+		params(Collections.singletonMap(name, value));
+		return this;
+	}
+
 	@SuppressWarnings({ "rawtypes", "unchecked" })
 	public SQLSelect<T> params(Map<String, ?> parameters) {
 		if (this.params == null) {
@@ -365,7 +444,9 @@ public class SQLSelect<T> extends IndirectQuery implements Select<T> {
 		template.setFetchOffset(offset);
 		template.setPageSize(pageSize);
 		template.setStatementFetchSize(statementFetchSize);
+		template.setQueryTimeout(queryTimeout);
 		template.setUseScalar(useScalar);
+		template.setResultMapper(resultMapper);
 
 		return template;
 	}
@@ -547,6 +628,19 @@ public class SQLSelect<T> extends IndirectQuery implements Select<T> {
 		return this;
 	}
 
+	/**
+	 * Sets query timeout
+	 * @since 4.2
+	 */
+	public SQLSelect<T> queryTimeout(int queryTimeout) {
+		if(this.queryTimeout != queryTimeout) {
+			this.queryTimeout = queryTimeout;
+			this.replacementQuery = null;
+		}
+
+		return this;
+	}
+
 	/**
 	 * @return JBDC statement's fetch size
 	 */
@@ -554,6 +648,10 @@ public class SQLSelect<T> extends IndirectQuery implements Select<T> {
 		return statementFetchSize;
 	}
 
+	public int getQueryTimeout() {
+		return queryTimeout;
+	}
+
 	/**
 	 * Merges a prefetch path with specified semantics into the query prefetch tree.
 	 * @param path Path expression
@@ -589,6 +687,26 @@ public class SQLSelect<T> extends IndirectQuery implements Select<T> {
 		return this;
 	}
 
+	/**
+	 * Map result of this query by processing with a given function.
+	 * <br/>
+	 * Could be used to map plain Object[] to some domain-specific object.
+	 * <br/>
+	 * <b>Note:</b> this method could be called multiple time, result will be mapped by all functions in the call order.
+	 * @param mapper function that maps result to the required type.
+	 * @return this query with changed result type
+	 * @param <E> new result type
+	 *
+	 * @see ColumnSelect#map(Function)
+	 *
+	 * @since 4.2
+	 */
+	@SuppressWarnings("unchecked")
+	public <E> SQLSelect<E> map(Function<T, E> mapper) {
+		this.resultMapper = mapper;
+		return (SQLSelect<E>)this;
+	}
+
 	@SuppressWarnings("unchecked")
 	private <E> SQLSelect<E> fetchingDataRows() {
 		this.isFetchingDataRows = true;
