@@ -28,13 +28,17 @@ import org.apache.cayenne.exp.ExpressionFactory;
 import org.apache.cayenne.log.JdbcEventLogger;
 import org.apache.cayenne.map.DbAttribute;
 import org.apache.cayenne.map.DbEntity;
+import org.apache.cayenne.query.ObjectSelect;
 import org.apache.cayenne.query.PrefetchTreeNode;
 import org.apache.cayenne.query.SelectQuery;
-import org.apache.cayenne.query.SortOrder;
 import org.apache.cayenne.test.jdbc.DBHelper;
+import org.apache.cayenne.testdo.testmap.ArtGroup;
 import org.apache.cayenne.testdo.testmap.Artist;
 import org.apache.cayenne.testdo.testmap.ArtistExhibit;
+import org.apache.cayenne.testdo.testmap.Award;
 import org.apache.cayenne.testdo.testmap.CompoundPainting;
+import org.apache.cayenne.testdo.testmap.Exhibit;
+import org.apache.cayenne.testdo.testmap.Gallery;
 import org.apache.cayenne.testdo.testmap.Painting;
 import org.apache.cayenne.unit.UnitDbAdapter;
 import org.apache.cayenne.unit.di.server.CayenneProjects;
@@ -42,6 +46,9 @@ import org.apache.cayenne.unit.di.server.ServerCase;
 import org.apache.cayenne.unit.di.server.UseServerRuntime;
 import org.junit.Test;
 
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Comparator;
 import java.util.Date;
 import java.util.List;
 
@@ -71,10 +78,10 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 	@Test
 	public void testCreateSqlString1() throws Exception {
 		// query with qualifier and ordering
-		SelectQuery<Artist> q = new SelectQuery<Artist>(Artist.class, ExpressionFactory.likeExp("artistName", "a%"));
-		q.addOrdering("dateOfBirth", SortOrder.ASCENDING);
+		ObjectSelect<Artist> q = ObjectSelect.query(Artist.class, Artist.ARTIST_NAME.like("a%"))
+				.orderBy(Artist.DATE_OF_BIRTH.asc());
 
-		DefaultSelectTranslator defaultSelectTranslator = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver());
+		SelectTranslator defaultSelectTranslator = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver());
 		String generatedSql = defaultSelectTranslator.getSql();
 
 		// do some simple assertions to make sure all parts are in
@@ -92,12 +99,11 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 	@Test
 	public void testDbEntityQualifier() throws Exception {
 
-		SelectQuery<Artist> q = new SelectQuery<Artist>(Artist.class);
+		ObjectSelect<Artist> q = ObjectSelect.query(Artist.class);
 		final DbEntity entity = context.getEntityResolver().getDbEntity("ARTIST");
 		final DbEntity middleEntity = context.getEntityResolver().getDbEntity("ARTIST_GROUP");
 
-		DefaultSelectTranslator transl = new DefaultSelectTranslator(q, dataNode.getAdapter(),
-				dataNode.getEntityResolver());
+		SelectTranslator transl = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver());
 
 		entity.setQualifier(ExpressionFactory.exp("ARTIST_NAME = \"123\""));
 		middleEntity.setQualifier(ExpressionFactory.exp("GROUP_ID = 1987"));
@@ -111,11 +117,11 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 			assertTrue(generatedSql.startsWith("SELECT "));
 			assertTrue(generatedSql.indexOf(" FROM ") > 0);
 			if (generatedSql.contains("RTRIM")) {
-				assertTrue(generatedSql.indexOf("ARTIST_NAME) = ") > generatedSql.indexOf("RTRIM("));
+				assertTrue(generatedSql.indexOf("ARTIST_NAME) =") > generatedSql.indexOf("RTRIM("));
 			} else if (generatedSql.contains("TRIM")) {
-				assertTrue(generatedSql.indexOf("ARTIST_NAME) = ") > generatedSql.indexOf("TRIM("));
+				assertTrue(generatedSql.indexOf("ARTIST_NAME) =") > generatedSql.indexOf("TRIM("));
 			} else {
-				assertTrue(generatedSql.indexOf("ARTIST_NAME = ") > 0);
+				assertTrue(generatedSql.indexOf("ARTIST_NAME =") > 0);
 			}
 		} finally {
 			entity.setQualifier(null);
@@ -126,14 +132,13 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 	@Test
 	public void testDbEntityQualifier_OuterJoin() throws Exception {
 
-		SelectQuery<Painting> q = SelectQuery.query(Painting.class);
-		q.addOrdering("toArtist+.artistName", SortOrder.ASCENDING);
+		ObjectSelect<Painting> q = ObjectSelect.query(Painting.class)
+				.orderBy(Painting.TO_ARTIST.outer().dot(Artist.ARTIST_NAME).asc());
 
 		final DbEntity entity = context.getEntityResolver().getDbEntity("ARTIST");
 		final DbEntity middleEntity = context.getEntityResolver().getDbEntity("ARTIST_GROUP");
 
-		DefaultSelectTranslator transl = new DefaultSelectTranslator(q, dataNode.getAdapter(),
-				dataNode.getEntityResolver());
+		SelectTranslator transl = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver());
 
 		entity.setQualifier(ExpressionFactory.exp("ARTIST_NAME = \"123\""));
 		middleEntity.setQualifier(ExpressionFactory.exp("GROUP_ID = 1987"));
@@ -147,11 +152,11 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 			assertTrue(generatedSql.startsWith("SELECT "));
 			assertTrue(generatedSql.indexOf(" FROM ") > 0);
 			if (generatedSql.contains("RTRIM")) {
-				assertTrue(generatedSql.indexOf("ARTIST_NAME) = ") > generatedSql.indexOf("RTRIM("));
+				assertTrue(generatedSql.indexOf("ARTIST_NAME) =") > generatedSql.indexOf("RTRIM("));
 			} else if (generatedSql.contains("TRIM")) {
-				assertTrue(generatedSql.indexOf("ARTIST_NAME) = ") > generatedSql.indexOf("TRIM("));
+				assertTrue(generatedSql.indexOf("ARTIST_NAME) =") > generatedSql.indexOf("TRIM("));
 			} else {
-				assertTrue(generatedSql.indexOf("ARTIST_NAME = ") > 0);
+				assertTrue(generatedSql.indexOf("ARTIST_NAME =") > 0);
 			}
 
 		} finally {
@@ -163,14 +168,12 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 	@Test
 	public void testDbEntityQualifier_FlattenedRel() throws Exception {
 
-		SelectQuery<Artist> q = new SelectQuery<Artist>(Artist.class, ExpressionFactory.matchExp("groupArray.name",
-				"bar"));
+		ObjectSelect<Artist> q = ObjectSelect.query(Artist.class, Artist.GROUP_ARRAY.dot(ArtGroup.NAME).eq("bar"));
 
 		final DbEntity entity = context.getEntityResolver().getDbEntity("ARTIST");
 		final DbEntity middleEntity = context.getEntityResolver().getDbEntity("ARTIST_GROUP");
 
-		DefaultSelectTranslator transl = new DefaultSelectTranslator(q, dataNode.getAdapter(),
-				dataNode.getEntityResolver());
+		SelectTranslator transl = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver());
 
 		entity.setQualifier(ExpressionFactory.exp("ARTIST_NAME = \"123\""));
 		middleEntity.setQualifier(ExpressionFactory.exp("GROUP_ID = 1987"));
@@ -184,11 +187,11 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 			assertTrue(generatedSql.startsWith("SELECT "));
 			assertTrue(generatedSql.indexOf(" FROM ") > 0);
 			if (generatedSql.contains("RTRIM")) {
-				assertTrue(generatedSql.indexOf("ARTIST_NAME) = ") > generatedSql.indexOf("RTRIM("));
+				assertTrue(generatedSql.indexOf("ARTIST_NAME) =") > generatedSql.indexOf("RTRIM("));
 			} else if (generatedSql.contains("TRIM")) {
-				assertTrue(generatedSql.indexOf("ARTIST_NAME) = ") > generatedSql.indexOf("TRIM("));
+				assertTrue(generatedSql.indexOf("ARTIST_NAME) =") > generatedSql.indexOf("TRIM("));
 			} else {
-				assertTrue(generatedSql.indexOf("ARTIST_NAME = ") > 0);
+				assertTrue(generatedSql.indexOf("ARTIST_NAME =") > 0);
 			}
 
 		} finally {
@@ -200,14 +203,12 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 	@Test
 	public void testDbEntityQualifier_RelatedMatch() throws Exception {
 
-		SelectQuery<Artist> q = new SelectQuery(Painting.class,
-				ExpressionFactory.matchExp("toArtist.artistName", "foo"));
+		ObjectSelect<Painting> q = ObjectSelect.query(Painting.class, Painting.TO_ARTIST.dot(Artist.ARTIST_NAME).eq("foo"));
 
 		final DbEntity entity = context.getEntityResolver().getDbEntity("ARTIST");
 		final DbEntity middleEntity = context.getEntityResolver().getDbEntity("ARTIST_GROUP");
 
-		DefaultSelectTranslator transl = new DefaultSelectTranslator(q, dataNode.getAdapter(),
-				dataNode.getEntityResolver());
+		SelectTranslator transl = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver());
 
 		entity.setQualifier(ExpressionFactory.exp("ARTIST_NAME = \"123\""));
 		middleEntity.setQualifier(ExpressionFactory.exp("GROUP_ID = 1987"));
@@ -221,11 +222,11 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 			assertTrue(generatedSql.startsWith("SELECT "));
 			assertTrue(generatedSql.indexOf(" FROM ") > 0);
 			if (generatedSql.contains("RTRIM")) {
-				assertTrue(generatedSql.indexOf("ARTIST_NAME) = ") > generatedSql.indexOf("RTRIM("));
+				assertTrue(generatedSql.indexOf("ARTIST_NAME) =") > generatedSql.indexOf("RTRIM("));
 			} else if (generatedSql.contains("TRIM")) {
-				assertTrue(generatedSql.indexOf("ARTIST_NAME) = ") > generatedSql.indexOf("TRIM("));
+				assertTrue(generatedSql.indexOf("ARTIST_NAME) =") > generatedSql.indexOf("TRIM("));
 			} else {
-				assertTrue(generatedSql.indexOf("ARTIST_NAME = ") > 0);
+				assertTrue(generatedSql.indexOf("ARTIST_NAME =") > 0);
 			}
 		} finally {
 			entity.setQualifier(null);
@@ -237,13 +238,13 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 	 * Tests query creation with "distinct" specified.
 	 */
 	@Test
+	@Deprecated
 	public void testCreateSqlString2() throws Exception {
 		// query with "distinct" set
-		SelectQuery q = new SelectQuery(Artist.class);
+		SelectQuery<Artist> q = new SelectQuery<>(Artist.class);
 		q.setDistinct(true);
 
-		String generatedSql = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver())
-				.getSql();
+		String generatedSql = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver()).getSql();
 
 		// do some simple assertions to make sure all parts are in
 		assertNotNull(generatedSql);
@@ -257,13 +258,16 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 	 */
 	@Test
 	public void testCreateSqlString5() throws Exception {
-		// query with qualifier and ordering
-		SelectQuery q = new SelectQuery(ArtistExhibit.class);
-		q.setQualifier(ExpressionFactory.likeExp("toArtist.artistName", "a%"));
-		q.andQualifier(ExpressionFactory.likeExp("toExhibit.toGallery.paintingArray.toArtist.artistName", "a%"));
+		ObjectSelect<ArtistExhibit> q = ObjectSelect.query(ArtistExhibit.class)
+				.where(ArtistExhibit.TO_ARTIST
+						.dot(Artist.ARTIST_NAME).like( "a%"))
+				.and(ArtistExhibit.TO_EXHIBIT
+						.dot(Exhibit.TO_GALLERY)
+						.dot(Gallery.PAINTING_ARRAY)
+						.dot(Painting.TO_ARTIST)
+						.dot(Artist.ARTIST_NAME).like("a%"));
 
-		String generatedSql = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver())
-				.getSql();
+		String generatedSql = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver()).getSql();
 		// logObj.warn("Query: " + generatedSql);
 
 		// do some simple assertions to make sure all parts are in
@@ -289,10 +293,9 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 	 */
 	@Test
 	public void testCreateSqlString6() throws Exception {
-		// query with qualifier and ordering
-		SelectQuery q = new SelectQuery(ArtistExhibit.class);
-		q.setQualifier(ExpressionFactory.likeExp("toArtist.artistName", "a%"));
-		q.andQualifier(ExpressionFactory.likeExp("toArtist.paintingArray.paintingTitle", "p%"));
+		ObjectSelect<ArtistExhibit> q = ObjectSelect.query(ArtistExhibit.class)
+				.where(ArtistExhibit.TO_ARTIST.dot(Artist.ARTIST_NAME).like("a%"))
+				.and(ArtistExhibit.TO_ARTIST.dot(Artist.PAINTING_ARRAY).dot(Painting.PAINTING_TITLE).like("p%"));
 
 		String generatedSql = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver())
 				.getSql();
@@ -318,9 +321,9 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 	 */
 	@Test
 	public void testCreateSqlString7() throws Exception {
-		SelectQuery q = new SelectQuery(Artist.class);
-		q.setQualifier(ExpressionFactory.greaterExp("dateOfBirth", new Date()));
-		q.andQualifier(ExpressionFactory.lessExp("dateOfBirth", new Date()));
+		ObjectSelect<Artist> q = ObjectSelect.query(Artist.class)
+				.where(Artist.DATE_OF_BIRTH.gt(new Date()))
+				.and(Artist.DATE_OF_BIRTH.lt(new Date()));
 
 		String generatedSql = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver())
 				.getSql();
@@ -351,13 +354,11 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 	 */
 	@Test
 	public void testCreateSqlString8() throws Exception {
-		SelectQuery q = new SelectQuery();
-		q.setRoot(Painting.class);
-		q.setQualifier(ExpressionFactory.greaterExp("toArtist.dateOfBirth", new Date()));
-		q.andQualifier(ExpressionFactory.lessExp("toArtist.dateOfBirth", new Date()));
+		ObjectSelect<Painting> q = ObjectSelect.query(Painting.class)
+				.where(Painting.TO_ARTIST.dot(Artist.DATE_OF_BIRTH).gt(new Date()))
+				.and(Painting.TO_ARTIST.dot(Artist.DATE_OF_BIRTH).lt(new Date()));
 
-		String generatedSql = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver())
-				.getSql();
+		String generatedSql = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver()).getSql();
 
 		// do some simple assertions to make sure all parts are in
 		assertNotNull(generatedSql);
@@ -379,7 +380,7 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 	@Test
 	public void testCreateSqlString9() throws Exception {
 		// query for a compound ObjEntity with qualifier
-		SelectQuery q = new SelectQuery(CompoundPainting.class, ExpressionFactory.likeExp("artistName", "a%"));
+		ObjectSelect<CompoundPainting> q = ObjectSelect.query(CompoundPainting.class, CompoundPainting.ARTIST_NAME.like("a%"));
 
 		String sql = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver()).getSql();
 
@@ -431,10 +432,9 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 	@Test
 	public void testCreateSqlString10() throws Exception {
 		// query with to-many joint prefetches
-		SelectQuery q = new SelectQuery(Artist.class);
-		q.addPrefetch(Artist.PAINTING_ARRAY.joint());
+		ObjectSelect<Artist> q = ObjectSelect.query(Artist.class).prefetch(Artist.PAINTING_ARRAY.joint());
 
-		DefaultSelectTranslator transl = new DefaultSelectTranslator(q, dataNode.getAdapter(),
+		SelectTranslator transl = new DefaultSelectTranslator(q, dataNode.getAdapter(),
 				dataNode.getEntityResolver());
 		String sql = transl.getSql();
 		assertNotNull(sql);
@@ -449,32 +449,31 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 		assertTrue(sql, sql.indexOf("PAINTING_ID") > 0);
 
 		// assert we have one join
-		assertEquals(1, transl.joinStack.size());
+		assertTrue(transl.hasJoins());
 	}
 
 	@Test
 	public void testCreateSqlString11() throws Exception {
 		// query with joint prefetches and other joins
-		SelectQuery q = new SelectQuery(Artist.class, ExpressionFactory.exp("paintingArray.paintingTitle = 'a'"));
-		q.addPrefetch(Artist.PAINTING_ARRAY.joint());
+		ObjectSelect q = ObjectSelect.query(Artist.class)
+				.where(Artist.PAINTING_ARRAY.dot(Painting.PAINTING_TITLE).eq("a"))
+				.prefetch(Artist.PAINTING_ARRAY.joint());
 
-		DefaultSelectTranslator transl = new DefaultSelectTranslator(q, dataNode.getAdapter(),
+		SelectTranslator transl = new DefaultSelectTranslator(q, dataNode.getAdapter(),
 				dataNode.getEntityResolver());
 
 		transl.getSql();
 
 		// assert we only have one join
-		assertEquals(2, transl.joinStack.size());
 		assertTrue(transl.hasJoins());
 	}
 
 	@Test
 	public void testCreateSqlString12() throws Exception {
 		// query with to-one joint prefetches
-		SelectQuery q = new SelectQuery(Painting.class);
-		q.addPrefetch(Painting.TO_ARTIST.joint());
+		ObjectSelect<Painting> q = ObjectSelect.query(Painting.class).prefetch(Painting.TO_ARTIST.joint());
 
-		DefaultSelectTranslator transl = new DefaultSelectTranslator(q, dataNode.getAdapter(),
+		SelectTranslator transl = new DefaultSelectTranslator(q, dataNode.getAdapter(),
 				dataNode.getEntityResolver());
 
 		String sql = transl.getSql();
@@ -490,16 +489,14 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 		assertTrue(sql, sql.indexOf("PAINTING_ID") > 0);
 
 		// assert we have one join
-		assertEquals(1, transl.joinStack.size());
 		assertTrue(transl.hasJoins());
 	}
 
 	@Test
 	public void testCreateSqlString13() throws Exception {
 		// query with invalid joint prefetches
-		SelectQuery q = new SelectQuery(Painting.class);
-		q.addPrefetch("invalid.invalid").setSemantics(PrefetchTreeNode.JOINT_PREFETCH_SEMANTICS);
-
+		ObjectSelect<Painting> q = ObjectSelect.query(Painting.class)
+				.prefetch("invalid.invalid", PrefetchTreeNode.JOINT_PREFETCH_SEMANTICS);
 		try {
 			new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver()).getSql();
 			fail("Invalid jointPrefetch must have thrown...");
@@ -512,10 +509,10 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 	public void testCreateSqlStringWithQuoteSqlIdentifiers() throws Exception {
 
 		try {
-			SelectQuery q = new SelectQuery(Artist.class);
+			ObjectSelect<Artist> q = ObjectSelect.query(Artist.class)
+					.orderBy(Artist.DATE_OF_BIRTH.asc());
 			DbEntity entity = context.getEntityResolver().getDbEntity("ARTIST");
 			entity.getDataMap().setQuotingSQLIdentifiers(true);
-			q.addOrdering("dateOfBirth", SortOrder.ASCENDING);
 
 			String charStart = unitAdapter.getIdentifiersStartQuote();
 			String charEnd = unitAdapter.getIdentifiersEndQuote();
@@ -549,11 +546,12 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 	public void testCreateSqlStringWithQuoteSqlIdentifiers2() throws Exception {
 
 		try {
-			SelectQuery q = new SelectQuery(Artist.class);
+			ObjectSelect<Artist> q = ObjectSelect.query(Artist.class)
+					.where(Artist.DATE_OF_BIRTH.gt(new Date()))
+					.and(Artist.DATE_OF_BIRTH.lt(new Date()));
+
 			DbEntity entity = context.getEntityResolver().getDbEntity("ARTIST");
 			entity.getDataMap().setQuotingSQLIdentifiers(true);
-			q.setQualifier(ExpressionFactory.greaterExp("dateOfBirth", new Date()));
-			q.andQualifier(ExpressionFactory.lessExp("dateOfBirth", new Date()));
 
 			String charStart = unitAdapter.getIdentifiersStartQuote();
 			String charEnd = unitAdapter.getIdentifiersEndQuote();
@@ -574,14 +572,12 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 			int iWhere = s.indexOf(" WHERE ");
 			assertTrue(iWhere > iArtist);
 
-			int dateOfBirth2 = s.indexOf(charStart + "t0" + charEnd + "." + charStart + "DATE_OF_BIRTH" + charEnd
-					+ " > ?");
+			int dateOfBirth2 = s.indexOf(charStart + "t0" + charEnd + "." + charStart + "DATE_OF_BIRTH" + charEnd + " > ?");
 			assertTrue(dateOfBirth2 > iWhere);
 
 			int iAnd = s.indexOf(" AND ");
 			assertTrue(iAnd > iWhere);
-			int dateOfBirth3 = s.indexOf(charStart + "t0" + charEnd + "." + charStart + "DATE_OF_BIRTH" + charEnd
-					+ " < ?");
+			int dateOfBirth3 = s.indexOf(charStart + "t0" + charEnd + "." + charStart + "DATE_OF_BIRTH" + charEnd + " < ?");
 			assertTrue(dateOfBirth3 > iAnd);
 
 		} finally {
@@ -596,8 +592,9 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 		// query with joint prefetches and other joins
 		// and with QuoteSqlIdentifiers = true
 		try {
-			SelectQuery q = new SelectQuery(Artist.class, ExpressionFactory.exp("paintingArray.paintingTitle = 'a'"));
-			q.addPrefetch(Artist.PAINTING_ARRAY.joint());
+			ObjectSelect<Artist> q = ObjectSelect.query(Artist.class)
+					.where(Artist.PAINTING_ARRAY.dot(Painting.PAINTING_TITLE).eq("a"))
+					.prefetch(Artist.PAINTING_ARRAY.joint());
 
 			DbEntity entity = context.getEntityResolver().getDbEntity("ARTIST");
 			entity.getDataMap().setQuotingSQLIdentifiers(true);
@@ -607,61 +604,60 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 
 			String s = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver()).getSql();
 
-			assertTrue(s.startsWith("SELECT DISTINCT "));
+			assertTrue(s, s.startsWith("SELECT DISTINCT "));
 			int iFrom = s.indexOf(" FROM ");
-			assertTrue(iFrom > 0);
+			assertTrue(s, iFrom > 0);
 			int artistName = s.indexOf(charStart + "t0" + charEnd + "." + charStart + "ARTIST_NAME" + charEnd);
-			assertTrue(artistName > 0 && artistName < iFrom);
+			assertTrue(s, artistName > 0 && artistName < iFrom);
 			int artistId = s.indexOf(charStart + "t0" + charEnd + "." + charStart + "ARTIST_ID" + charEnd);
-			assertTrue(artistId > 0 && artistId < iFrom);
+			assertTrue(s, artistId > 0 && artistId < iFrom);
 			int dateOfBirth = s.indexOf(charStart + "t0" + charEnd + "." + charStart + "DATE_OF_BIRTH" + charEnd);
-			assertTrue(dateOfBirth > 0 && dateOfBirth < iFrom);
+			assertTrue(s, dateOfBirth > 0 && dateOfBirth < iFrom);
 			int estimatedPrice = s.indexOf(charStart + "t1" + charEnd + "." + charStart + "ESTIMATED_PRICE" + charEnd);
-			assertTrue(estimatedPrice > 0 && estimatedPrice < iFrom);
+			assertTrue(s, estimatedPrice > 0 && estimatedPrice < iFrom);
 			int paintingDescription = s.indexOf(charStart + "t1" + charEnd + "." + charStart + "PAINTING_DESCRIPTION"
 					+ charEnd);
-			assertTrue(paintingDescription > 0 && paintingDescription < iFrom);
+			assertTrue(s, paintingDescription > 0 && paintingDescription < iFrom);
 			int paintingTitle = s.indexOf(charStart + "t1" + charEnd + "." + charStart + "PAINTING_TITLE" + charEnd);
-			assertTrue(paintingTitle > 0 && paintingTitle < iFrom);
+			assertTrue(s, paintingTitle > 0 && paintingTitle < iFrom);
 			int artistIdT1 = s.indexOf(charStart + "t1" + charEnd + "." + charStart + "ARTIST_ID" + charEnd);
-			assertTrue(artistIdT1 > 0 && artistIdT1 < iFrom);
+			assertTrue(s, artistIdT1 > 0 && artistIdT1 < iFrom);
 			int galleryId = s.indexOf(charStart + "t1" + charEnd + "." + charStart + "GALLERY_ID" + charEnd);
-			assertTrue(galleryId > 0 && galleryId < iFrom);
+			assertTrue(s, galleryId > 0 && galleryId < iFrom);
 			int paintingId = s.indexOf(charStart + "t1" + charEnd + "." + charStart + "PAINTING_ID" + charEnd);
-			assertTrue(paintingId > 0 && paintingId < iFrom);
+			assertTrue(s, paintingId > 0 && paintingId < iFrom);
 			int iArtist = s.indexOf(charStart + "ARTIST" + charEnd + " " + charStart + "t0" + charEnd);
-			assertTrue(iArtist > iFrom);
+			assertTrue(s, iArtist > iFrom);
 			int iLeftJoin = s.indexOf("LEFT JOIN");
-			assertTrue(iLeftJoin > iFrom);
+			assertTrue(s, iLeftJoin > iFrom);
 			int iPainting = s.indexOf(charStart + "PAINTING" + charEnd + " " + charStart + "t1" + charEnd);
-			assertTrue(iPainting > iLeftJoin);
+			assertTrue(s, iPainting > iLeftJoin);
 			int iOn = s.indexOf(" ON ");
-			assertTrue(iOn > iLeftJoin);
+			assertTrue(s, iOn > iLeftJoin);
 			int iArtistId = s.indexOf(charStart + "t0" + charEnd + "." + charStart + "ARTIST_ID" + charEnd, iLeftJoin);
-			assertTrue(iArtistId > iOn);
+			assertTrue(s, iArtistId > iOn);
 			int iArtistIdT1 = s
 					.indexOf(charStart + "t1" + charEnd + "." + charStart + "ARTIST_ID" + charEnd, iLeftJoin);
-			assertTrue(iArtistIdT1 > iOn);
+			assertTrue(s, iArtistIdT1 > iOn);
 			int i = s.indexOf("=", iLeftJoin);
-			assertTrue(iArtistIdT1 > i || iArtistId > i);
+			assertTrue(s, iArtistIdT1 > i || iArtistId > i);
 			int iJoin = s.indexOf("JOIN");
-			assertTrue(iJoin > iLeftJoin);
+			assertTrue(s, iJoin > iLeftJoin);
 			int iPainting2 = s.indexOf(charStart + "PAINTING" + charEnd + " " + charStart + "t2" + charEnd);
-			assertTrue(iPainting2 > iJoin);
+			assertTrue(s, iPainting2 > iJoin);
 			int iOn2 = s.indexOf(" ON ");
-			assertTrue(iOn2 > iJoin);
+			assertTrue(s, iOn2 > iJoin);
 			int iArtistId2 = s.indexOf(charStart + "t0" + charEnd + "." + charStart + "ARTIST_ID" + charEnd, iJoin);
-			assertTrue(iArtistId2 > iOn2);
+			assertTrue(s, iArtistId2 > iOn2);
 			int iArtistId2T2 = s.indexOf(charStart + "t2" + charEnd + "." + charStart + "ARTIST_ID" + charEnd, iJoin);
-			assertTrue(iArtistId2T2 > iOn2);
+			assertTrue(s, iArtistId2T2 > iOn2);
 			int i2 = s.indexOf("=", iJoin);
-			assertTrue(iArtistId2T2 > i2 || iArtistId2 > i2);
+			assertTrue(s, iArtistId2T2 > i2 || iArtistId2 > i2);
 			int iWhere = s.indexOf(" WHERE ");
-			assertTrue(iWhere > iJoin);
+			assertTrue(s, iWhere > iJoin);
 
-			int paintingTitle2 = s.indexOf(charStart + "t2" + charEnd + "." + charStart + "PAINTING_TITLE" + charEnd
-					+ " = ?");
-			assertTrue(paintingTitle2 > iWhere);
+			int paintingTitle2 = s.indexOf(charStart + "t2" + charEnd + "." + charStart + "PAINTING_TITLE" + charEnd + " = ?");
+			assertTrue(s, paintingTitle2 > iWhere);
 
 		} finally {
 			DbEntity entity = context.getEntityResolver().getDbEntity("ARTIST");
@@ -675,8 +671,7 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 		// query with to-one joint prefetches
 		// and with QuoteSqlIdentifiers = true
 		try {
-			SelectQuery q = new SelectQuery(Painting.class);
-			q.addPrefetch(Painting.TO_ARTIST.joint());
+			ObjectSelect<Painting> q = ObjectSelect.query(Painting.class).prefetch(Painting.TO_ARTIST.joint());
 
 			DbEntity entity = context.getEntityResolver().getDbEntity("PAINTING");
 			entity.getDataMap().setQuotingSQLIdentifiers(true);
@@ -686,45 +681,45 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 
 			String s = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver()).getSql();
 
-			assertTrue(s.startsWith("SELECT "));
+			assertTrue(s, s.startsWith("SELECT "));
 			int iFrom = s.indexOf(" FROM ");
-			assertTrue(iFrom > 0);
+			assertTrue(s, iFrom > 0);
 
 			int paintingDescription = s.indexOf(charStart + "t0" + charEnd + "." + charStart + "PAINTING_DESCRIPTION"
 					+ charEnd);
-			assertTrue(paintingDescription > 0 && paintingDescription < iFrom);
+			assertTrue(s, paintingDescription > 0 && paintingDescription < iFrom);
 			int paintingTitle = s.indexOf(charStart + "t0" + charEnd + "." + charStart + "PAINTING_TITLE" + charEnd);
-			assertTrue(paintingTitle > 0 && paintingTitle < iFrom);
+			assertTrue(s, paintingTitle > 0 && paintingTitle < iFrom);
 			int artistIdT1 = s.indexOf(charStart + "t0" + charEnd + "." + charStart + "ARTIST_ID" + charEnd);
-			assertTrue(artistIdT1 > 0 && artistIdT1 < iFrom);
+			assertTrue(s, artistIdT1 > 0 && artistIdT1 < iFrom);
 			int estimatedPrice = s.indexOf(charStart + "t0" + charEnd + "." + charStart + "ESTIMATED_PRICE" + charEnd);
-			assertTrue(estimatedPrice > 0 && estimatedPrice < iFrom);
+			assertTrue(s, estimatedPrice > 0 && estimatedPrice < iFrom);
 			int galleryId = s.indexOf(charStart + "t0" + charEnd + "." + charStart + "GALLERY_ID" + charEnd);
-			assertTrue(galleryId > 0 && galleryId < iFrom);
+			assertTrue(s, galleryId > 0 && galleryId < iFrom);
 			int paintingId = s.indexOf(charStart + "t0" + charEnd + "." + charStart + "PAINTING_ID" + charEnd);
-			assertTrue(paintingId > 0 && paintingId < iFrom);
+			assertTrue(s, paintingId > 0 && paintingId < iFrom);
 			int artistName = s.indexOf(charStart + "t1" + charEnd + "." + charStart + "ARTIST_NAME" + charEnd);
-			assertTrue(artistName > 0 && artistName < iFrom);
+			assertTrue(s, artistName > 0 && artistName < iFrom);
 			int artistId = s.indexOf(charStart + "t1" + charEnd + "." + charStart + "ARTIST_ID" + charEnd);
-			assertTrue(artistId > 0 && artistId < iFrom);
+			assertTrue(s, artistId > 0 && artistId < iFrom);
 			int dateOfBirth = s.indexOf(charStart + "t1" + charEnd + "." + charStart + "DATE_OF_BIRTH" + charEnd);
-			assertTrue(dateOfBirth > 0 && dateOfBirth < iFrom);
+			assertTrue(s, dateOfBirth > 0 && dateOfBirth < iFrom);
 			int iPainting = s.indexOf(charStart + "PAINTING" + charEnd + " " + charStart + "t0" + charEnd);
-			assertTrue(iPainting > iFrom);
+			assertTrue(s, iPainting > iFrom);
 
 			int iLeftJoin = s.indexOf("LEFT JOIN");
-			assertTrue(iLeftJoin > iFrom);
+			assertTrue(s, iLeftJoin > iFrom);
 			int iArtist = s.indexOf(charStart + "ARTIST" + charEnd + " " + charStart + "t1" + charEnd);
-			assertTrue(iArtist > iLeftJoin);
+			assertTrue(s, iArtist > iLeftJoin);
 			int iOn = s.indexOf(" ON ");
-			assertTrue(iOn > iLeftJoin);
+			assertTrue(s, iOn > iLeftJoin);
 			int iArtistId = s.indexOf(charStart + "t0" + charEnd + "." + charStart + "ARTIST_ID" + charEnd, iLeftJoin);
-			assertTrue(iArtistId > iOn);
+			assertTrue(s, iArtistId > iOn);
 			int iArtistIdT1 = s
 					.indexOf(charStart + "t1" + charEnd + "." + charStart + "ARTIST_ID" + charEnd, iLeftJoin);
-			assertTrue(iArtistIdT1 > iOn);
+			assertTrue(s, iArtistIdT1 > iOn);
 			int i = s.indexOf("=", iLeftJoin);
-			assertTrue(iArtistIdT1 > i || iArtistId > i);
+			assertTrue(s, iArtistIdT1 > i || iArtistId > i);
 
 		} finally {
 			DbEntity entity = context.getEntityResolver().getDbEntity("PAINTING");
@@ -737,16 +732,28 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 	 */
 	@Test
 	public void testBuildResultColumns1() throws Exception {
-		SelectQuery q = new SelectQuery(Painting.class);
-		DefaultSelectTranslator tr = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver());
+		ObjectSelect<Painting> q = ObjectSelect.query(Painting.class);
+		SelectTranslator tr = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver());
 
-		List<?> columns = tr.buildResultColumns();
+		tr.getSql();
+
+		List<ColumnDescriptor> columns = Arrays.asList(tr.getResultColumns());
+		columns.sort(Comparator.comparing(ColumnDescriptor::getName));
+
+		DbEntity entity = context.getEntityResolver().getDbEntity("PAINTING");
+		List<DbAttribute> attributes = new ArrayList<>(entity.getAttributes());
+		attributes.sort(Comparator.comparing(DbAttribute::getName));
 
 		// all DbAttributes must be included
-		DbEntity entity = context.getEntityResolver().getDbEntity("PAINTING");
-		for (final DbAttribute a : entity.getAttributes()) {
-			ColumnDescriptor c = new ColumnDescriptor(a, "t0");
-			assertTrue("No descriptor for " + a + ", columns: " + columns, columns.contains(c));
+		assertEquals(attributes.size(), columns.size());
+
+		for(int i=0; i<attributes.size(); i++) {
+			DbAttribute attribute = attributes.get(i);
+			ColumnDescriptor descriptor = columns.get(i);
+			assertEquals(attribute, descriptor.getAttribute());
+			assertEquals(attribute.getName(), descriptor.getName());
+			assertEquals(attribute.getName(), descriptor.getDataRowKey());
+			assertEquals(attribute.getType(), descriptor.getJdbcType());
 		}
 	}
 
@@ -755,32 +762,98 @@ public class DefaultSelectTranslatorIT extends ServerCase {
 	 */
 	@Test
 	public void testBuildResultColumns2() throws Exception {
-		SelectQuery q = new SelectQuery(Painting.class);
-		q.addPrefetch(Painting.TO_ARTIST.joint());
-		DefaultSelectTranslator tr = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver());
+		ObjectSelect<Painting> q = ObjectSelect.query(Painting.class).prefetch(Painting.TO_ARTIST.joint());
+		SelectTranslator tr = new DefaultSelectTranslator(q, dataNode.getAdapter(), dataNode.getEntityResolver());
 
-		List<?> columns = tr.buildResultColumns();
+		tr.getSql();
 
-		// assert root entity columns
-		DbEntity entity = context.getEntityResolver().getDbEntity("PAINTING");
-		for (final DbAttribute a : entity.getAttributes()) {
-			ColumnDescriptor c = new ColumnDescriptor(a, "t0");
-			assertTrue("No descriptor for " + a + ", columns: " + columns, columns.contains(c));
-		}
+		List<ColumnDescriptor> columns = Arrays.asList(tr.getResultColumns());
+		columns.sort(Comparator.comparing(ColumnDescriptor::getName));
 
-		// assert joined columns
-		DbEntity joined = context.getEntityResolver().getDbEntity("ARTIST");
-		for (final DbAttribute a : joined.getAttributes()) {
+		DbEntity rootEntity = context.getEntityResolver().getDbEntity("PAINTING");
+		List<DbAttribute> attributes = new ArrayList<>(rootEntity.getAttributes());
+		DbEntity joinedEntity = context.getEntityResolver().getDbEntity("ARTIST");
+		attributes.addAll(joinedEntity.getAttributes());
+		attributes.sort(Comparator.comparing(DbAttribute::getName));
 
-			// skip ARTIST PK, it is joined from painting
-			if (Artist.ARTIST_ID_PK_COLUMN.equals(a.getName())) {
-				continue;
+		for(int i=0; i<attributes.size(); i++) {
+			DbAttribute attribute = attributes.get(i);
+			ColumnDescriptor descriptor = columns.get(i);
+			assertEquals(attribute, descriptor.getAttribute());
+			assertEquals(attribute.getName(), descriptor.getName());
+			if("ARTIST".equals(attribute.getEntity().getName())) {
+				assertEquals("toArtist." + attribute.getName(), descriptor.getDataRowKey());
+			} else {
+				assertEquals(attribute.getName(), descriptor.getDataRowKey());
 			}
-
-			ColumnDescriptor c = new ColumnDescriptor(a, "t1");
-			c.setDataRowKey("toArtist." + a.getName());
-			assertTrue("No descriptor for " + a + ", columns: " + columns, columns.contains(c));
+			assertEquals(attribute.getType(), descriptor.getJdbcType());
 		}
 	}
 
+	@Test
+	public void testAliasedJoins() {
+		ObjectSelect<Artist> query = ObjectSelect.query(Artist.class)
+				.where(ExpressionFactory.and(
+						ExpressionFactory.matchAllExp("|awardArray", 1, 2),
+						Artist.AWARD_ARRAY.alias("aw1").dot(Award.NAME).eq("123")
+				));
+		query.select(context);
+
+		DefaultSelectTranslator translator = new DefaultSelectTranslator(query, dataNode.getAdapter(),
+																		 context.getEntityResolver());
+		translator.translate();
+
+		int totalJoins = translator.getContext().getTableCount() - 1;
+		assertEquals(3, totalJoins);
+	}
+
+	@Test
+	public void testAliasedJoins_DifferentAliases() {
+		ObjectSelect<Artist> query = ObjectSelect.query(Artist.class)
+				.where(ExpressionFactory.and(
+						Artist.AWARD_ARRAY.alias("aw1").containsId(1),
+						Artist.AWARD_ARRAY.alias("aw2").dot(Award.NAME).eq("123")
+				));
+		query.select(context);
+
+		DefaultSelectTranslator translator = new DefaultSelectTranslator(query, dataNode.getAdapter(),
+																		 context.getEntityResolver());
+		translator.translate();
+
+		int totalJoins = translator.getContext().getTableCount() - 1;
+		assertEquals(2, totalJoins);
+	}
+
+	@Test
+	public void testAliasedJoins_DifferentProperties() {
+		ObjectSelect<Artist> query = ObjectSelect.query(Artist.class)
+				.where(ExpressionFactory.and(
+						Artist.AWARD_ARRAY.alias("aw1").containsId(1),
+						Artist.PAINTING_ARRAY.alias("aw2").dot(Painting.PAINTING_TITLE).eq("123")
+				));
+		query.select(context);
+
+		DefaultSelectTranslator translator = new DefaultSelectTranslator(query, dataNode.getAdapter(),
+																		 context.getEntityResolver());
+		translator.translate();
+
+		int totalJoins = translator.getContext().getTableCount() - 1;
+		assertEquals(2, totalJoins);
+	}
+
+	@Test
+	public void testAliasedJoins_FlattenedRelationship() {
+		ObjectSelect<Artist> query = ObjectSelect.query(Artist.class)
+				.where(ExpressionFactory.and(
+						ExpressionFactory.matchAllExp("|groupArray.name", "ag1", "ag2")
+				));
+		query.select(context);
+
+		DefaultSelectTranslator translator = new DefaultSelectTranslator(query, dataNode.getAdapter(),
+																		 context.getEntityResolver());
+		translator.translate();
+
+		int totalJoins = translator.getContext().getTableCount() - 1;
+		assertEquals(4, totalJoins);
+	}
 }
