@@ -21,12 +21,14 @@ package org.apache.cayenne.dba;
 
 import org.apache.cayenne.MockSerializable;
 import org.apache.cayenne.di.Inject;
+import org.apache.cayenne.map.DbAttribute;
 import org.apache.cayenne.unit.di.server.CayenneProjects;
 import org.apache.cayenne.unit.di.server.ServerCase;
 import org.apache.cayenne.unit.di.server.ServerCaseDataSourceFactory;
 import org.apache.cayenne.unit.di.server.UseServerRuntime;
 import org.junit.Test;
 
+import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.sql.Connection;
 import java.sql.DatabaseMetaData;
@@ -100,6 +102,32 @@ public class TypesMappingIT extends ServerCase {
         assertEquals(Types.BIGINT, TypesMapping.getSqlTypeByJava(Long.TYPE));
     }
 
+    @Test
+    public void testDecimalType() {
+        DbAttribute attribute = new DbAttribute("test");
+        attribute.setType(Types.DECIMAL);
+
+        attribute.setScale(0);
+        attribute.setMaxLength(38);
+        assertEquals(BigInteger.class.getName(), TypesMapping.getJavaBySqlType(attribute));
+
+        attribute.setScale(0);
+        attribute.setMaxLength(5);
+        assertEquals(Integer.class.getName(), TypesMapping.getJavaBySqlType(attribute));
+
+        attribute.setScale(0);
+        attribute.setMaxLength(10);
+        assertEquals(Long.class.getName(), TypesMapping.getJavaBySqlType(attribute));
+
+        attribute.setScale(1);
+        attribute.setMaxLength(5);
+        assertEquals(BigDecimal.class.getName(), TypesMapping.getJavaBySqlType(attribute));
+
+        attribute.setScale(5);
+        attribute.setMaxLength(38);
+        assertEquals(BigDecimal.class.getName(), TypesMapping.getJavaBySqlType(attribute));
+    }
+
     @Test
     public void testTypeInfoCompleteness() throws Exception {
         // check counts
