@@ -37,6 +37,7 @@ import org.apache.cayenne.dbsync.reverse.dbimport.IncludeTable;
 import org.apache.cayenne.dbsync.reverse.dbimport.PatternParam;
 import org.apache.cayenne.dbsync.reverse.dbimport.ReverseEngineering;
 import org.apache.cayenne.dbsync.reverse.dbimport.Schema;
+import org.apache.cayenne.dbsync.reverse.dbimport.SchemaContainer;
 import org.apache.cayenne.modeler.ClassLoadingService;
 import org.apache.cayenne.modeler.dialog.db.load.DbImportTreeNode;
 import org.apache.cayenne.modeler.pref.DBConnectionInfo;
@@ -45,22 +46,21 @@ public class DatabaseSchemaLoader {
 
     private static final String INCLUDE_ALL_PATTERN = "%";
 
-    private ReverseEngineering databaseReverseEngineering;
+    private final ReverseEngineering databaseReverseEngineering;
 
     public DatabaseSchemaLoader() {
         databaseReverseEngineering = new ReverseEngineering();
     }
 
     public ReverseEngineering load(DBConnectionInfo connectionInfo,
-                                   ClassLoadingService loadingService,
-                                   String[] tableTypesFromConfig) throws Exception {
+                                   ClassLoadingService loadingService) throws Exception {
         DbAdapter dbAdapter = connectionInfo.makeAdapter(loadingService);
         try (Connection connection = connectionInfo.makeDataSource(loadingService).getConnection()) {
-            String[] types = tableTypesFromConfig != null && tableTypesFromConfig.length != 0 ?
-                    tableTypesFromConfig :
-                    new String[]{"TABLE", "VIEW", "SYSTEM TABLE",
-                            "GLOBAL TEMPORARY", "LOCAL TEMPORARY", "ALIAS", "SYNONYM"};
-            processCatalogs(connection, types, dbAdapter);
+            processCatalogs(connection, dbAdapter);
+        }
+
+        if (databaseReverseEngineering.getSchemas().isEmpty() && databaseReverseEngineering.getCatalogs().isEmpty()) {
+            loadTables(connectionInfo, loadingService, null, null);
         }
 
         sort();
@@ -82,46 +82,91 @@ public class DatabaseSchemaLoader {
         filterContainer.getIncludeProcedures().sort(comparator);
     }
 
-    private void processCatalogs(Connection connection, String[] types, DbAdapter dbAdapter) throws SQLException {
+    private void processCatalogs(Connection connection, DbAdapter dbAdapter) throws SQLException {
         try (ResultSet rsCatalog = connection.getMetaData().getCatalogs()) {
             boolean hasCatalogs = false;
             List<String> systemCatalogs = dbAdapter.getSystemCatalogs();
-            while (rsCatalog.next()) {
+            while (rsCatalog.next() && dbAdapter.supportsCatalogsOnReverseEngineering()) {
                 hasCatalogs = true;
                 String catalog = rsCatalog.getString("TABLE_CAT");
-                if(!systemCatalogs.contains(catalog)) {
-                    processSchemas(connection, types, catalog, dbAdapter);
+                if (!systemCatalogs.contains(catalog)) {
+                    processSchemas(connection, catalog, dbAdapter);
                 }
             }
-            if(!hasCatalogs) {
-                processSchemas(connection, types, null, dbAdapter);
+            if (!hasCatalogs) {
+                processSchemas(connection, null, dbAdapter);
             }
         }
     }
 
     private void processSchemas(Connection connection,
-                                String[] types,
                                 String catalog,
                                 DbAdapter dbAdapter) throws SQLException {
         DatabaseMetaData metaData = connection.getMetaData();
-        try(ResultSet rsSchema = metaData.getSchemas(catalog, null)) {
-            boolean hasSchemas = false;
-            List<String> systemSchemas = dbAdapter.getSystemSchemas();
-            while (rsSchema.next()) {
-                hasSchemas = true;
-                String schema = rsSchema.getString("TABLE_SCHEM");
-                if(!systemSchemas.contains(schema)) {
-                    ResultSet resultSet = metaData.getTables(catalog, schema, INCLUDE_ALL_PATTERN, types);
-                    packTable(resultSet);
-                    packProcedures(connection);
+        boolean hasSchemas = false;
+        if (metaData.supportsSchemasInTableDefinitions()) {
+            try (ResultSet rsSchema = metaData.getSchemas(catalog, null)) {
+                List<String> systemSchemas = dbAdapter.getSystemSchemas();
+                while (rsSchema.next()) {
+                    hasSchemas = true;
+                    String schema = rsSchema.getString("TABLE_SCHEM");
+                    if (!systemSchemas.contains(schema)) {
+                        packFilterContainer(catalog, schema);
+                    }
                 }
             }
-            if(!hasSchemas) {
-                ResultSet resultSet = metaData.getTables(catalog, null, INCLUDE_ALL_PATTERN, types);
-                packTable(resultSet);
+        }
+
+        if (catalog != null && !hasSchemas) {
+            packFilterContainer(catalog, null);
+        }
+    }
+
+    public ReverseEngineering loadTables(DBConnectionInfo connectionInfo,
+                                         ClassLoadingService loadingService,
+                                         TreePath path,
+                                         String[] tableTypesFromConfig) throws SQLException {
+        int pathIndex = 1;
+        String catalogName = null, schemaName = null;
+
+        Object userObject = getUserObjectOrNull(path, pathIndex);
+        if (userObject != null) {
+            if (userObject instanceof Catalog) {
+                Catalog catalog = (Catalog) userObject;
+                catalogName = catalog.getName();
+                if (!catalog.getSchemas().isEmpty()) {
+                    userObject = getUserObjectOrNull(path, ++pathIndex);
+                    if (userObject instanceof Schema) {
+                        schemaName = ((Schema) userObject).getName();
+                    }
+                }
+            } else if (userObject instanceof Schema) {
+                schemaName = ((Schema) userObject).getName();
+            }
+        }
+
+        try (Connection connection = connectionInfo.makeDataSource(loadingService).getConnection()) {
+            String[] types = tableTypesFromConfig != null && tableTypesFromConfig.length != 0 ?
+                    tableTypesFromConfig :
+                    new String[]{"TABLE", "VIEW", "SYSTEM TABLE",
+                            "GLOBAL TEMPORARY", "LOCAL TEMPORARY", "ALIAS", "SYNONYM"};
+            try (ResultSet resultSet = connection.getMetaData()
+                    .getTables(catalogName, schemaName, INCLUDE_ALL_PATTERN, types)) {
+                boolean hasTables = false;
+                while (resultSet.next()) {
+                    hasTables = true;
+                    String table = resultSet.getString("TABLE_NAME");
+                    String schema = resultSet.getString("TABLE_SCHEM");
+                    String catalog = resultSet.getString("TABLE_CAT");
+                    packTable(table, catalog == null ? catalogName : catalog, schema, null);
+                }
+                if (!hasTables && (catalogName != null || schemaName != null)) {
+                    packFilterContainer(catalogName, schemaName);
+                }
                 packProcedures(connection);
             }
         }
+        return databaseReverseEngineering;
     }
 
     public ReverseEngineering loadColumns(DBConnectionInfo connectionInfo,
@@ -130,17 +175,14 @@ public class DatabaseSchemaLoader {
         int pathIndex = 1;
         String catalogName = null, schemaName = null;
 
-        Object userObject = getUserObject(path, pathIndex);
-        if(userObject instanceof Catalog) {
+        Object userObject = getUserObjectOrNull(path, pathIndex);
+        if (userObject instanceof Catalog) {
             catalogName = ((Catalog) userObject).getName();
-            userObject = getUserObject(path, ++pathIndex);
-            if(userObject instanceof Schema) {
-                schemaName = ((Schema) userObject).getName();
-                userObject = getUserObject(path, ++pathIndex);
-            }
-        } else if(userObject instanceof Schema) {
+            userObject = getUserObjectOrNull(path, ++pathIndex);
+        }
+        if (userObject instanceof Schema) {
             schemaName = ((Schema) userObject).getName();
-            userObject = getUserObject(path, ++pathIndex);
+            userObject = getUserObjectOrNull(path, ++pathIndex);
         }
 
         String tableName = processTable(userObject);
@@ -156,36 +198,65 @@ public class DatabaseSchemaLoader {
         return databaseReverseEngineering;
     }
 
-    private Object getUserObject(TreePath path, int pathIndex) {
-        return ((DbImportTreeNode)path.getPathComponent(pathIndex)).getUserObject();
+    private FilterContainer packFilterContainer(String catalogName, String schemaName) {
+        SchemaContainer parentCatalog;
+        if (catalogName == null) {
+            parentCatalog = databaseReverseEngineering;
+        } else {
+            parentCatalog = getCatalogByName(databaseReverseEngineering.getCatalogs(), catalogName);
+            if (parentCatalog == null) {
+                parentCatalog = new Catalog();
+                parentCatalog.setName(catalogName);
+                databaseReverseEngineering.addCatalog((Catalog) parentCatalog);
+            }
+        }
+
+        Schema parentSchema = null;
+        if (schemaName != null) {
+            parentSchema = getSchemaByName(parentCatalog.getSchemas(), schemaName);
+            if (parentSchema == null) {
+                parentSchema = new Schema();
+                parentSchema.setName(schemaName);
+                parentCatalog.addSchema(parentSchema);
+            }
+        }
+
+        return parentSchema == null ? parentCatalog : parentSchema;
+    }
+
+    private Object getUserObjectOrNull(TreePath path, int pathIndex) {
+        if (path == null) {
+            return null;
+        }
+        return ((DbImportTreeNode) path.getPathComponent(pathIndex)).getUserObject();
     }
 
     private String processTable(Object userObject) {
-        if(userObject instanceof IncludeTable) {
-            return  ((IncludeTable) userObject).getPattern();
+        if (userObject instanceof IncludeTable) {
+            return ((IncludeTable) userObject).getPattern();
         }
         return null;
     }
 
     private void packProcedures(Connection connection) throws SQLException {
         Collection<Catalog> catalogs = databaseReverseEngineering.getCatalogs();
-        for(Catalog catalog : catalogs) {
+        for (Catalog catalog : catalogs) {
             Collection<Schema> schemas = catalog.getSchemas();
-            if(!schemas.isEmpty()) {
-                for(Schema schema : schemas) {
+            if (!schemas.isEmpty()) {
+                for (Schema schema : schemas) {
                     ResultSet procResultSet = getProcedures(connection, catalog.getName(), schema.getName());
-                    packFunction(procResultSet, schema);
+                    packProcedures(procResultSet, schema);
                 }
             } else {
                 ResultSet procResultSet = getProcedures(connection, catalog.getName(), null);
-                packFunction(procResultSet, catalog);
+                packProcedures(procResultSet, catalog);
             }
         }
 
         Collection<Schema> schemas = databaseReverseEngineering.getSchemas();
-        for(Schema schema : schemas) {
+        for (Schema schema : schemas) {
             ResultSet procResultSet = getProcedures(connection, null, schema.getName());
-            packFunction(procResultSet, schema);
+            packProcedures(procResultSet, schema);
         }
     }
 
@@ -193,7 +264,7 @@ public class DatabaseSchemaLoader {
         return connection.getMetaData().getProcedures(catalog, schema, "%");
     }
 
-    private void packFunction(ResultSet resultSet, FilterContainer filterContainer) throws SQLException {
+    private void packProcedures(ResultSet resultSet, FilterContainer filterContainer) throws SQLException {
         while (resultSet.next()) {
             IncludeProcedure includeProcedure =
                     new IncludeProcedure(resultSet.getString("PROCEDURE_NAME"));
@@ -203,15 +274,6 @@ public class DatabaseSchemaLoader {
         }
     }
 
-    private void packTable(ResultSet resultSet) throws SQLException {
-        while (resultSet.next()) {
-            String tableName = resultSet.getString("TABLE_NAME");
-            String schemaName = resultSet.getString("TABLE_SCHEM");
-            String catalogName = resultSet.getString("TABLE_CAT");
-            packTable(tableName, catalogName, schemaName, null);
-        }
-    }
-
     private void packTable(String tableName, String catalogName, String schemaName, String columnName) {
         IncludeTable table = new IncludeTable();
         table.setPattern(tableName);
@@ -220,49 +282,11 @@ public class DatabaseSchemaLoader {
             if (!databaseReverseEngineering.getIncludeTables().contains(table)) {
                 databaseReverseEngineering.addIncludeTable(table);
             }
+            addColumn(null, table, columnName);
             return;
         }
 
-        FilterContainer filterContainer;
-        if (catalogName != null && schemaName == null) {
-            Catalog parentCatalog = getCatalogByName(databaseReverseEngineering.getCatalogs(), catalogName);
-
-            if(parentCatalog == null) {
-                parentCatalog = new Catalog();
-                parentCatalog.setName(catalogName);
-                databaseReverseEngineering.addCatalog(parentCatalog);
-            }
-            filterContainer = parentCatalog;
-        } else if (catalogName == null) {
-            Schema parentSchema = getSchemaByName(databaseReverseEngineering.getSchemas(), schemaName);
-
-            if(parentSchema == null) {
-                parentSchema = new Schema();
-                parentSchema.setName(schemaName);
-                databaseReverseEngineering.addSchema(parentSchema);
-            }
-            filterContainer = parentSchema;
-        } else {
-            Catalog parentCatalog = getCatalogByName(databaseReverseEngineering.getCatalogs(), catalogName);
-            Schema parentSchema;
-            if (parentCatalog != null) {
-                parentSchema = getSchemaByName(parentCatalog.getSchemas(), schemaName);
-                if(parentSchema == null) {
-                    parentSchema = new Schema();
-                    parentSchema.setName(schemaName);
-                    parentCatalog.addSchema(parentSchema);
-                }
-            } else {
-                parentCatalog = new Catalog();
-                parentCatalog.setName(catalogName);
-                parentSchema = new Schema();
-                parentSchema.setName(schemaName);
-                parentCatalog.addSchema(parentSchema);
-                databaseReverseEngineering.addCatalog(parentCatalog);
-            }
-            filterContainer = parentSchema;
-        }
-
+        FilterContainer filterContainer = packFilterContainer(catalogName, schemaName);
         addTable(filterContainer, table);
         addColumn(filterContainer, table, columnName);
     }
@@ -274,38 +298,35 @@ public class DatabaseSchemaLoader {
     }
 
     private void addColumn(FilterContainer filterContainer, IncludeTable table, String columnName) {
+        if (columnName == null) {
+            return;
+        }
+
+        filterContainer = filterContainer == null ? databaseReverseEngineering : filterContainer;
         IncludeTable foundTable = getTableByName(filterContainer.getIncludeTables(), table.getPattern());
         table = foundTable != null ? foundTable : table;
-        if (columnName != null) {
-            IncludeColumn includeColumn = new IncludeColumn(columnName);
-            table.addIncludeColumn(includeColumn);
-        }
+        IncludeColumn includeColumn = new IncludeColumn(columnName);
+        table.addIncludeColumn(includeColumn);
     }
 
     private Catalog getCatalogByName(Collection<Catalog> catalogs, String catalogName) {
-        for (Catalog catalog : catalogs) {
-            if (catalog.getName().equals(catalogName)) {
-                return catalog;
-            }
-        }
-        return null;
+        return catalogs.stream()
+                .filter(catalog -> catalog.getName().equals(catalogName))
+                .findAny()
+                .orElse(null);
     }
 
     private IncludeTable getTableByName(Collection<IncludeTable> tables, String catalogName) {
-        for (IncludeTable table : tables) {
-            if (table.getPattern().equals(catalogName)) {
-                return table;
-            }
-        }
-        return null;
+        return tables.stream()
+                .filter(table -> table.getPattern().equals(catalogName))
+                .findAny()
+                .orElse(null);
     }
 
     private Schema getSchemaByName(Collection<Schema> schemas, String schemaName) {
-        for (Schema schema : schemas) {
-            if (schema.getName().equals(schemaName)) {
-                return schema;
-            }
-        }
-        return null;
+        return schemas.stream()
+                .filter(schema -> schema.getName().equals(schemaName))
+                .findAny()
+                .orElse(null);
     }
 }
